/*
 ***** LICENSE *****
 */

/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


/*
 ***** TODO *****
 */

// - Feature: Convert usleep in nanosleep (more safe in modern multithreaded systems)
// - Feature: Use robot / Use board - Don't move robor / Don't update views (fast game) + Debug [en/dis]abled via checkbox
// - Feature: Coords transformation layer from Game-Space to Stepper-Space
// - Feature: Flag "need_refresh" in actor_game (for no-refresh misses - buggy?)
// - Feature: Load / Save / Play-Fwd-Rew match
// - Feature: Undo Button
// - Bug: Find bug in compute_valid_moves() and remove trick (valid move (0,0) not really permitted)
// - Bug: "History" in standard code
// - Bug: Wrong history colors and texts when in 0-players-mode and with random clicks on the chacboard
// - Bug: SIGSEGV closing gui during autoplay
// - Bug: SIGSEGV near the end of the second autoplay (really difficult to catch) 
// - Bug: SIGSEGV on boot (random - rare)
// - Bug: Stop cronographs on checkmate
// - Bug: Typical open move with Q agaist K: K can't move and loose, but he could recover with the sacrifice of B or T 
// - Bug: Call tower "R" (Rook - english) and not "T" (Tower - italian) - knight in "H" (Horse) and not "K" (Knight) 'cause it's not King
// - Bug: in 1 player mode the robot must move half times and not always
// - Bug: history wrong colors and text when 2-players-match bacome autoplay
// - Bug: reset after autoplay don't clear history (eraseable with manual reset, correct?), but not other stats
// - Bug: cumpute_valid_move() calculate warn-moves and put high-colors, then actor_game() removes them and loose right colors
// - SUPER BUG: good_move not done swap players!
// - SUPER BUG: Arrocco with K not in the right place creates a T and another K and swap players (in 1-player-mode ?) (try to inject) (rewrite arrocco !!)
// - Elegance: Actual player [0..1] in standard game, [1..2] in autoplay (in autoplay stats actual_player color swapped)
// - CheckMate: you can't put yourself in it !!
// - PROPEDEUTICO: verify il attack is really safe, not only il you loose the piece (checkmate excluded)
// - PROPEDENTICO2: write a function that aggisn a weight to the bestmove (it needs bigger arrays for the rating number)
// - PROPEDEUTICO3: write a function that assign a risk factor to every chackboard situation for every player (or two functions: risk factor, opportunity)
// - Redraw chackboard when choosing number of players
// - Bug: Clicking a "number of players" during the match it destroys the history

// - gpk_backup_moves() ==> gpk_backup_best_moves();
// - gpk_backup_moves2() ==> gpk_backup_valid_moves();
// - gpk_recovery_moves() ==> etc
// - gpk_recovery_moves2() ==> etc


/*
 ***** CHANGELOG *****
 */

// Ver 0.1.7a
// - Bugfix: No more SIGSEGV on lpt0 access on modern machines without lpt-port (now removed because I/O will be made with Arduino ASAP)
// - Bugfix: Internal state is shown traslated (converted from internal numeric value to a more understandable text)
// - Bugfix: Curl lower timeout: NO MORE PROBLEMS with actual timeouts using last libcurl/modern linux distributions
// - Bugfix: BIG Memory-leak removed: gtk_style copyed without free() (250Mb per minute in autoplay, now ZERO !!)
// - Network Lag Monitor (via curl statistics)
// - Nickname generated with username+PID
// - Todo-list translation completed
// - Minor documantation updates
// - Minor code cleanups (various)

// Ver 0.1.6a
// - New state machine (play with 1 click, not 2 clicks)
// - AI Algorithm aggressive but not suicide ;)
// - Intra match and intra computation stats
// - Joshua can finish match when checkmate in 1 move
// - About Dialog-Box
// - Strategy used when help checked + better state machine integration
// - When a game starts, other player buttons are disabled
// - Removed pointer passed to each function to trach GUI data structure
// - Debug: New dump functions + tuning on old functions + messages in english + colored boards
// - Bugfix: Minor autoplay bugs
// - Bugfix: Remove a lot of unused variables
// - Bugfix: Better indent code
// - Bugfix: Swapped deprecated gdk_pixmap_unref() with g_object_unref()
// - License: Now Joshua is GNU GPL v3 !!

// Ver 0.1.5a
// - Tactical CheckBoard Redesigned
// - gtk_rc_parse("/etc/joshua.rc") removed (theme independent color managment)
// - AI Algorithm more aggressive (!)
// - Disabled Buttons NOT sensitive
// - Debug checkboard injectable for developers ('adjust' button)
// - Added function to move the king under check
// - Bugfix: history colors were swapped
// - Bugfix: minor bugfixes in gpk_compute_best_moves()
// - Bugfix: signals disabled in libcurl (caused segmentation fault when dns was unreachable) 
// - Bugfix: redraw refreshed only one graphic window 
// - Bugfix: robot tower elevation effect resync 

// Ver 0.1.4a
// - Parallel port accesser to drive steppers + gui monitor (beta test)
// - Message-Server on gipoco.com + player list downloaded by the server (new thread)
// - Coments and GUI Translation from Italian to English 
// - Gui tuning (stepper bits renumeration, controls restyle, chat GUI, window not resizeable, autoreset don't clear history)
// - PANIC Button implementation (Game Pause)
// - Basic players match timers (total and for each player)
// - Bugfix: closing GUI message thread don't crash (perfect exit)
// - Bugfix: history shows moves in alternate colors without warnings in terminal
// - Bugfix: convert sprintf code in snprintf + parser updated + strlen wrapper
// - Bugfix: pawns jumped chips the first time they moved
// - Bugfix: old buggy reset after autoplay now works (asyncronows & threaded)
// - BugFix: state machine debug, players swapped in autoplay
// - BugFix: no more need to run joshua and reset (gpk_game_reset() in main.c without errors)

// Ver 0.1.3a
// - Big GUI restyle and new functions added
// - Removed old camera view
// - gtk_rc file pointed in /etc
// - Initial setup of the robot BETA (gui buttons not funtionals)
// - Minor bugfixes, types, estetics, removed old commented code
// - New message thread (show and blink) 
// - Bugfix: chips tanken strange behaviour in depot


/*
 ***** INCLUDE HEADERS *****
 */

#include <sys/io.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <libgnomeui-2.0/gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

#include <gtk/gtk.h>


/*
 ***** DEFINE IO PORTS *****
 */

#define base_addr1 0x378           /* lpt1 port base address */
#define base_addr2 0x378           /* lpt2 port base address */
#define base_addr3 0x378           /* lpt3 port base address */
#define base_addr4 0x378           /* lpt4 port base address */


/*
 ***** DEFINE DELAYS *****
 */

#define DELAY_CURL_LOCK                      24988
#define DELAY_PRE_COMPUTE                    12345
#define DELAY_PRE_COMPUTE2                   12345
#define DELAY_SHOW_OTHER_MOVE               980000

// SLOW TIMING
#define DELAY_THREAD_WAIT_MOVE               23256
#define DELAY_THREAD_WAIT_MOVE2              23256
#define DELAY_THREAD_WAIT_MOVE3              23256
#define DELAY_THREAD_BETWEEN_MOVE            63231
#define DELAY_THREAD_BETWEEN_PLAYERS        131313
#define DELAY_THREAD2_BETWEEN_STATES         13777
#define DELAY_THREAD2_BETWEEN_STATES2        88888
#define DELAY_THREAD2_BETWEEN_STATES_SLOW   500000

/*
// FAST TIMING
#define DELAY_THREAD_WAIT_MOVE                 236
#define DELAY_THREAD_WAIT_MOVE2                236
#define DELAY_THREAD_WAIT_MOVE3                236
#define DELAY_THREAD_BETWEEN_MOVE              631
#define DELAY_THREAD_BETWEEN_PLAYERS          1313
#define DELAY_THREAD2_BETWEEN_STATES           177
#define DELAY_THREAD2_BETWEEN_STATES2          888
#define DELAY_THREAD2_BETWEEN_STATES_SLOW     5000
*/

#define DELAY_THREAD3_BETWEEN_HILIGHT_SLOW  975000
#define DELAY_THREAD3_BETWEEN_HILIGHT_FAST   80000
#define DELAY_THREAD4_NETWORK_WAIT         2998377


/*
 ***** DEFINE COLORS *****
 */

#define BLACK 0
#define RED 1
#define GREEN 2
#define BLUE 3
#define YELLOW 4
#define ORANGE 5
#define GOLD 6
#define SEA_GREEN 7
#define AQUAMARINE 8
#define NAVY_BLUE 9
#define LIGHT_TAN 10
#define LEMON 11
#define PURPLE 12
#define LIGHT_GREY 13
#define MID_GREY 14
#define MID2_GREY 15
#define DARK_GREY 16
#define RED_2 17
#define NAVY_BLUE_2 18
#define NAVY_BLUE_DARK 19
#define RED_DARK 20
#define WHITE 21


/*
 ***** FUNCTIONS DECLARATIONS *****
 */

void gpk_update_chat(gchar *ttt);
void gpk_clear_chat(void);
void gpk_update_history(gchar *ttt);
void gpk_clear_history(void);
void gpk_game_reset(GtkButton *button);
void gpk_disable_buttons(void);
void gpk_dump_chessboard(void);
void gpk_dump_valid_chips(void);
void gpk_dump_valid_moves(void);
void gpk_dump_best_moves(void);

void gpk_update_chessboard(void);
void gpk_update_stat_num(int num, int val);
void gpk_update_stat_str(int num, char *str);
void gpk_update_stat_net_traf(int num);
void gpk_update_stat_net_lag(double *num);
void gpk_update_stats(void);

void gpk_actor_game(int x, int y);
void gpk_set_checkboard(int x, int y, gchar val, int colore);
void gpk_update_num_chips(int who, gchar type);
void gpk_check_gameover(void);
void gpk_refresh_colors_after_hilight(void);
void gpk_automata(void);
void gpk_do_arrocco(int who);
int  gpk_find_chips(int actual_player);
int  gpk_check_warning_position(int x, int y);
int  gpk_check_king_locked(int x, int y);

void gpk_backup_colors(void);
void gpk_backup_colors2(void);
void gpk_backup_board(void);
void gpk_backup_moves(void);
void gpk_backup_moves2(void);
void gpk_alternate_colors(void);
void gpk_recovery_colors(void);
void gpk_recovery_colors2(void);
void gpk_recovery_board(void);
void gpk_recovery_moves(void);
void gpk_recovery_moves2(void);

void gpk_insert_best_move(int x, int y);
int  gpk_compute_best_moves(int x, int y);
void gpk_insert_valid_move(int x, int y);
int  gpk_compute_valid_moves(int x, int y);
int  gpk_check_this_move(int x, int y);
void gpk_choose_best_move(void);

void gpk_outb(int val, int motor);
void gpk_curl_fetch(gchar *ttt, int debug);
void gpk_curl_free(int debug);
int  gpk_strlen(char *str);

void gpk_draw_position2(void);
void gpk_draw_position(void);
void gpk_set_color(GtkWidget *widget, GdkGC *gc, gint name);
void gpk_clear_community_treeview(void);
void gpk_populate_community_treeview(void);
void gpk_inject_board(void);

void *gpk_thread(void *arg);  // ----- thread autoplay -----
void *gpk_thread2(void *arg); // ----- thread robot-mv -----
void *gpk_thread3(void *arg); // ----- thread messages -----
void *gpk_thread4(void *arg); // ----- thread network -----


/*
 ***** VARIABLES DECLARATIONS *****
 */

GtkWidget *here;  // Main Window Pointer 

int ct;           // Chip taken by the robot (color) 
gchar tt;         // Chip taken by the robot (type)
gchar bt[8][12];  // Chips on backup checkboard    (8x8 chips + aux poiters for castling)
gchar gt[8][12];  // Chips on strategic checkboard (8x8 chips + aux poiters for castling)
gchar gt1[8][12]; // Chips on real-time checkboard (8x8 chips + aux poiters for castling)
gchar dt[2][16];  // Chips on depot (chips taken by each player)
int bc2[8][12];   // Chips colors on backup checkboard v2 (8x8 chips + aux pointers for castling)
int bc[8][12];    // Chips colors on backup checkboard    (8x8 chips + aux pointers for castling)
int gc[8][12];    // Chips colors on strategic checkboard (8x8 chips + aux pointers for castling)
int gc1[8][12];   // Chips colors on real-time checkboard (8x8 chips + aux pointers for castling)
int gc2[8][12];   // chips colors on temporary checkboard (for possible moves)
int gn1[8];       // Chips counter PLAYER 1
int gn2[8];       // Chips counter PLAYER 2
int vittorie[2];  // Win counter
int mangiate[2];  // Chips taken counter

int stato=0;         // Main State Machine
int timer_on=0;      // Start update timers ?
int timer[2];        // Timer for each player
int mossa[2];        // Move temp storage
int num_mosse=0;     // Move Counter
int actual_player=1; // Who is playing now ?
int arrocco_flag[2]; // Castling possible (k e t not moved) ?
int arrocco_started; // Castling started (wait request) ?
int game_paused=0;   // Game paused now ?
int warm_start=0;    // Game needs reset ?
int cold_start=1;    // Game just started ?

int his_positions[16][2];   // All positions of the other player
int best_move[28][2];       // All capturing moves of actual player
int valid_move[28][2];      // All moves of actual player
int valid_chip[28][2];      // All chips of actual player 
int best_move_cnt=0;        // Number of capturing moves from actual position
int valid_move_cnt=0;       // Number of validated moves from actual position
int valid_chip_cnt=0;       // Number of chips of actual player 
int his_positions_cnt;      // Number of chips of the other player

int temp_best_move[28][2];  // Copy of all capturing moves of actual player
int temp_valid_move[28][2]; // Copy of all moves of actual player
int temp_valid_chip[28][2]; // Copy of all chips of actual player 
int temp_best_move_cnt=0;   // Copy of number of capturing moves from actual position
int temp_valid_move_cnt=0;  // Copy of number of validated moves from actual position
int temp_valid_chip_cnt=0;  // Copy of number of chips on the checkboard 

int stato_r=0;         // Robot state machine
int intra_match=0;     // Match played between moves to choose best move
int intra_compute=0;   // Match played between moves to choose best move
int translate_bit[24]; // Bit-a-bit translation of axis state
int x1,y1,x2,y2;       // Robot start==>stop coordinates (on stepper space coord.)
int xx,yy,zz;          // Robot hand position in 3 dim. (on stepper space coord.)
int pre_x,pre_y;       // Previous move backup coords
int xo=9;              // Offset position top-left checkboard (x coord.)
int yo=7;              // Offset position top-left checkboard (y coord.)
int zo=7;              // Offset to reach chips from the z=0 plane (from up)
int xd[2];             // First free position of the chips depot (x coord.)
int yd[2];             // First free position of the chips depot (y coord.)

int tag_flag=0;                   // History color tags created ?
int lpt_perm_taken=0;             // I/O Permission taken ?
int message_hilight=0;            // Messages blinking ?
int message_blink_threshold=0;    // Fast blink countdown
gchar message_string[32];         // Blinking message
gchar status_translation[32][32]; // Blinking message

int thread_started=0;  // Main Thread started ?
int thread2_started=0; // Bot Thread started ?
int thread3_started=0; // Msg Thread started ?
int thread4_started=0; // Net Thread started ?
pthread_t thd1;        // PThread Structures for AUTOPLAY
pthread_t thd2;        // PThread Structures for GUI DRAW
pthread_t thd3;        // PThread Structures for MESSAGES
pthread_t thd4;        // PThread Structures for NETWORK

int need_refresh_list=0; // Reload needed for network player's list ?
int need_refresh_gui=0;  // Reload needed for tactical checkboard ?
int network_traf=0;      // Network traffic in bytes (only download)
double *network_lag;     // Network access time (xml download time)

static GdkPixmap *pixmap=NULL; // Robot preview draw-area
static GtkTextTag *tag;        // History red tag
static GtkTextTag *tag2;       // History blue tag

int gpk_curl_lock=0;   // Curl thread_safer lock
CURL *curl_handle;     // Curl handler
struct MemoryStruct {  // Curl buffer type
	char *memory;
	size_t size;
};
struct MemoryStruct chunk; // Curl buffer

enum {		           // Community View Columns
  COL_NAME = 0,
  COL_AGE,
  NUM_COLS
} ;

char *username; // User running Joshua (after randomization, it's used for logging into the community)


// Tactical Checkboard Colors
GdkColor col_lgrey;
GdkColor col_red;
GdkColor col_black;
GdkColor col_yellow;
GdkColor col_white;
GdkColor col_purple;
GdkColor col_green;
GdkColor col_orange;
GdkColor col_cyan;
GdkColor col_grey;


/*
 ***** CUSTOM FUNCTIONS *****
 */

/* ***** ASSIGN COLOR TO GRAPHIC CONTEST (for drawareas) ***** */
void gpk_set_color(GtkWidget *widget, GdkGC *gc2, gint name) {

	GdkColor colour;
	
	switch (name) {
	case BLACK:
		colour.red = 0;
		colour.green = 0;
		colour.blue = 0;
	break;
	case RED:
		colour.red = 65535;
		colour.green = 0;
		colour.blue = 0;
	break;
	case GREEN:
		colour.red = 0;
		colour.green = 65535;
		colour.blue = 0;
	break;
	case BLUE:
		colour.red = 0;
		colour.green = 0;
		colour.blue = 65535;
	break;
	case YELLOW:
		colour.red = 65535;
		colour.green = 65535;
		colour.blue = 0;
	break;
	case ORANGE:
	case GOLD:
		colour.red = 65535;
		colour.green = 50000;
		colour.blue = 10000;
	break;
	case SEA_GREEN:
		colour.red = 11150;
		colour.green = 55705;
		colour.blue = 40632;
	break;
	case AQUAMARINE:
		colour.red = 10000;
		colour.green = 65535;
		colour.blue = 50000;
	break;
	case NAVY_BLUE:
		colour.red = 0;
		colour.green = 32000;
		colour.blue = 65535;
	break;
	case LIGHT_TAN:
		colour.red = 65535;
		colour.green = 65535;
		colour.blue = 53739;
	break;
	case LEMON:
		colour.red = 64890;
		colour.green = 63569;
		colour.blue = 29491;
	break;
	case PURPLE:
		colour.red = 64000;
		colour.green = 30000;
		colour.blue = 64000;
	break;
	case LIGHT_GREY:
		colour.red = 58000;
		colour.green = 58000;
		colour.blue = 58000;
	break;
	case MID_GREY:
		colour.red = 40000;
		colour.green = 40000;
		colour.blue = 40000;
	break;
	case MID2_GREY:
		colour.red = 48000;
		colour.green = 48000;
		colour.blue = 48000;
	break;
	case DARK_GREY:
		colour.red = 16000;
		colour.green = 16000;
		colour.blue = 16000;
	break;
	case RED_2:
		colour.red = 65535;
		colour.green = 32000;
		colour.blue = 32000;
	break;
	case NAVY_BLUE_2:
		colour.red = 22000;
		colour.green = 48000;
		colour.blue = 65535;
	break;
	case NAVY_BLUE_DARK:
		colour.red = 0;
		colour.green = 16000;
		colour.blue = 32000;
	break;
	case RED_DARK:
		colour.red = 48000;
		colour.green = 0;
		colour.blue = 0;
	break;
	case WHITE:
	default:
		colour.red = 65535;
		colour.green = 65535;
		colour.blue = 65535;
	break;
	}

	if (!gdk_color_alloc(gtk_widget_get_colormap(widget), &colour)) g_print("Warning: colour assignment failed.\n");
	gdk_gc_set_foreground(gc2, &colour);
}


/* ***** Secure StrLen Wrapper ***** */
int gpk_strlen(char *str) {
	if (str!=NULL) {
		return strlen(str);
	} else {
		return 0;
	}
}


/* ***** Secure ReAlloc Wrapper ***** */
void *myrealloc(void *ptr, size_t size) {
	if (ptr) {
		return realloc(ptr, size);
	} else {
		return malloc(size);
	}
}


/* ***** Curl Memory Writer ***** */
size_t WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data) {
	
	size_t realsize=size *nmemb;
	struct MemoryStruct *mem=(struct MemoryStruct *)data;
	
	mem->memory=(char *)myrealloc(mem->memory, mem->size+realsize+1);
	if (mem->memory) {
		memcpy(&(mem->memory[mem->size]), ptr, realsize);
		mem->size+=realsize;
		mem->memory[mem->size]=0;
	}
	return realsize;
}


/* ***** Clear network player's list ***** */
void gpk_clear_community_treeview(void) {

	GtkTreeModel *model;
	GtkWidget *treeview=lookup_widget(GTK_WIDGET(here), "treeview1");

	model=gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	gtk_list_store_clear((GtkListStore *)model);
}


/* ***** Fetch xml from messageserver in http with curl and populate player's list ***** */
void gpk_populate_community_treeview(void) {

	gchar msg[64101];    // + 101 x debug overhead
	gchar temp[64001];   // + 1 x \n & string terminator
	gchar temp2[64001];  // + 1 x \n & string terminator
	gchar temp3[64];     // Nickname
	gchar nick[16];      // Must be 12
	gchar stat[8];       // Must be 4
	GtkTreeModel *model;
	GtkTreeIter new_row;
	GtkWidget *treeview=lookup_widget(GTK_WIDGET(here), "treeview1");
	int i,n;
	int cnt=0;

	
	msg[64100]=0;
	temp[64000]=0;
	temp2[64000]=0;
	temp3[64]=0;

	username=getenv("USER");
	if (gpk_strlen(username)>0) {
        	snprintf(temp3, 63, "%s-%d", username, (int)getpid());
	} else {
        	snprintf(temp3, 63, "Temp-%d", (int)getpid());
	}
       	gpk_update_stat_str(90, temp3);
	gpk_curl_fetch("http://joshua.gipoco.com/msgserv.php?cmd=list", 1);
	
	if (chunk.size==0) {
		snprintf(temp, 64000, "VOID !!\n");
	} else if (chunk.size>=64000) {
		snprintf(temp, 64000, "OVERFLOW !!\n");
	} else {
		snprintf(temp, chunk.size+1, "%s\n", chunk.memory);
	}
	
	temp[chunk.size]=0;
	for (i=0;i<64000;i++) { temp2[i]=0; }
	
	if (temp!=NULL) {
		n=gpk_strlen(temp);
		while (n>12) {
			for (i=0;i<16;i++) { nick[i]=0; }
			for (i=0;i<8;i++) { stat[i]=0; }
			snprintf(msg, 64100, "%d=parse('%s')\n", n, temp);
			g_print (msg);
			for (i=0;i<64000;i++) { temp2[i]=0; }
			for (i=0;i<(n-4);i++) {                        // Drop '252' at the beginnig of the message
				temp2[i]=temp[i+4];
			}
			temp2[i+1]=0;
			for (i=0;i<4;i++) {                            // Get 4 chars of player state
				stat[i]=temp2[i];
			}
			for (i=0;i<64000;i++) { temp[i]=0; }
			for (i=0;i<(gpk_strlen(temp2)-5);i++) {        // Reset temp vars to new offset (jump over the space)
				temp[i]=temp2[i+5];
			}
			temp[i+1]=0;
			for (i=0;((temp[i]!='<')&&(i<15));i++) {       // Get the link of actual pointer to '<' of '<br>' (field separator)
				nick[i]=temp[i];
				cnt=i;
			}
			for (i=0;i<64000;i++) { temp2[i]=0; }
			for (i=0;i<(gpk_strlen(temp)-(cnt+5));i++) {   // Reset temp vars to new offset (jump over the space)
				temp2[i]=temp[i+cnt+5];
			}
			temp2[i+1]=0;
			for (i=0;i<64000;i++) { temp[i]=0; }
			for (i=0;i<gpk_strlen(temp2);i++) {            // Reset temp vars to new offset (jump over the space)
				temp[i]=temp2[i];
			}
			temp[i+1]=0;
			model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
			gtk_list_store_append(GTK_LIST_STORE(model), &new_row);
			gtk_list_store_set(GTK_LIST_STORE(model), &new_row, COL_NAME, nick, COL_AGE, stat, -1);
			n=gpk_strlen(temp);
		}
	}
	gpk_curl_free(1);
	if (temp!=NULL) { 
		g_print(temp);
	}
}


/* ***** OUTPUT a VALUE on a MOTOR AXIS (OutB Wrapper) ***** */
void gpk_outb(int val, int motor) {

	GtkToggleButton *widget_a1=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton1");
	GtkToggleButton *widget_a2=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton2");
	GtkToggleButton *widget_a3=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton3");
	GtkToggleButton *widget_a4=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton4");

	GtkToggleButton *widget_b1=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton5");
	GtkToggleButton *widget_b2=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton6");
	GtkToggleButton *widget_b3=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton7");
	GtkToggleButton *widget_b4=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton8");

	GtkToggleButton *widget_c1=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton9");
	GtkToggleButton *widget_c2=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton10");
	GtkToggleButton *widget_c3=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton11");
	GtkToggleButton *widget_c4=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton12");

	GtkToggleButton *widget_d1=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton13");
	GtkToggleButton *widget_d2=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton14");
	GtkToggleButton *widget_d3=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton15");
	GtkToggleButton *widget_d4=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton16");

	if (motor==0) {
		if (lpt_perm_taken) outb(val, base_addr1);
		gtk_toggle_button_set_active(widget_a1, (((val&1)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_a2, (((val&2)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_a3, (((val&4)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_a4, (((val&8)>0)?TRUE:FALSE));
	}
	if (motor==1) {
		//if (lpt_perm_taken) outb(val, base_addr2);
		gtk_toggle_button_set_active(widget_b1, (((val&1)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_b2, (((val&2)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_b3, (((val&4)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_b4, (((val&8)>0)?TRUE:FALSE));
	}
	if (motor==2) {
		//if (lpt_perm_taken) outb(val, base_addr3);
		gtk_toggle_button_set_active(widget_c1, (((val&1)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_c2, (((val&2)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_c3, (((val&4)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_c4, (((val&8)>0)?TRUE:FALSE));
	}
	if (motor==3) {
		//if (lpt_perm_taken) outb(val, base_addr4);
		gtk_toggle_button_set_active(widget_d1, (((val&1)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_d2, (((val&2)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_d3, (((val&4)>0)?TRUE:FALSE));
		gtk_toggle_button_set_active(widget_d4, (((val&8)>0)?TRUE:FALSE));
	}
}


/* ***** DRAW BACKGROUND AND CHECKBOARD IN DRAWING-AREA 2 ***** */
void gpk_draw_position2(void) {

	int i;
	GtkWidget *widget=lookup_widget(GTK_WIDGET(here), "drawingarea2");
	GdkGC *gc0=widget->style->black_gc;

	// ----- FREE OLD + MALLOC FRAMEBUFFER -----
	if (pixmap) {
		//g_print("Unref 1\n");
		//gdk_pixmap_unref(pixmap);
		g_object_unref(pixmap);
	}
	pixmap=gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
	
	// ----- CLEAR BACKBGROUND + CHECKBOARD BACKGROUND -----
	gpk_set_color(widget, gc0, WHITE);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 0, 0, widget->allocation.width, widget->allocation.height);
	gpk_set_color(widget, gc0, MID2_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 100, 150, 80, 10);
	gpk_set_color(widget, gc0, MID_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 25, 160, 220, 5);
	gpk_set_color(widget, gc0, MID_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 35, 165, 20, 70);
	gpk_set_color(widget, gc0, MID_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 210, 165, 20, 70);

	// ----- DRAW CHECKBOARD -----
	for (i=101;i<180;i+=20) {
		gpk_set_color(widget, gc0, RED_DARK);
		gdk_draw_rectangle(pixmap, gc0, TRUE, i,    151, 8, 3);
		gpk_set_color(widget, gc0, WHITE);
		gdk_draw_rectangle(pixmap, gc0, TRUE, i+10, 151, 8, 3);
	}

	// ----- DRAW X e Y ROBOT AXIS -----
	gpk_set_color(widget, gc0, BLACK);
	gdk_draw_line(pixmap, gc0, 15+xx*10, 90+zz*4, 15+xx*10, 120+zz*4);
	gpk_set_color(widget, gc0, BLACK);
	gdk_draw_line(pixmap, gc0, 10, 115, 253, 115);
	gpk_set_color(widget, gc0, BLACK);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 14+xx*10, 119+zz*4, 3, 3);
	gpk_set_color(widget, gc0, ORANGE);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 13+xx*10-(zz/2), 119+zz*4, zz*2, zz);

	// ----- UPDATE FRAMEBUFFER -----
	GdkRectangle update_rect;
	update_rect.x = 8;
	update_rect.y = 3;
	update_rect.width = 243;
	update_rect.height = 3;
	gtk_widget_draw (widget, &update_rect);
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap, 8, 5, 5, 0, widget->allocation.width, widget->allocation.height);

	// ----- FREE -----	
	//g_object_unref(pixmap);   // MALFUNCTIONING
	//gdk_pixmap_unref(pixmap); // MALFUNCTIONING + DEPRECATED
}


/* ***** DRAW BACKGROUND AND CHECKBOARD IN DRAWING-AREA ***** */
void gpk_draw_position(void) {

	int i,j;
	int chip_width;
	GtkWidget *widget=lookup_widget(GTK_WIDGET((GtkWidget *)here), "drawingarea1");
	GdkGC *gc0=widget->style->black_gc;

	// ----- FREE OLD + MALLOC FRAMEBUFFER -----
	if (pixmap) {
		//g_print("Unref 2\n");
		//gdk_pixmap_unref(pixmap);  // DEPRECATED
		g_object_unref(pixmap);
	}
	pixmap=gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
	
	// ----- CLEAR BACKBGROUND + CHECKBOARD BACKGROUND + DEPOTS BACKGROUND -----
	gpk_set_color(widget, gc0, WHITE);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 0, 0, widget->allocation.width, widget->allocation.height);
	gpk_set_color(widget, gc0, MID2_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 25, 25, 220, 200);
	gpk_set_color(widget, gc0, LIGHT_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 100, 80, 80, 80);
	gpk_set_color(widget, gc0, LIGHT_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 100, 40, 80, 20);
	gpk_set_color(widget, gc0, LIGHT_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 100, 180, 80, 20);

	// ----- DRAW CHECKBOARD -----
	gpk_set_color(widget, gc0, BLACK);
	for (j=0;j<4;j++) {
		for (i=100;i<180;i+=20) {
			gdk_draw_rectangle(pixmap, gc0, TRUE, i,    (80+(j*20)), 10, 10);
			gdk_draw_rectangle(pixmap, gc0, TRUE, i+10, (90+(j*20)), 10, 10);
		}
	}

	// ----- DRAW CHIP'S DEPOTS -----
	gpk_set_color(widget, gc0, NAVY_BLUE_DARK);
	for (i=100;i<180;i+=20) {
		gdk_draw_rectangle(pixmap, gc0, TRUE, i,    40, 10, 10);
		gdk_draw_rectangle(pixmap, gc0, TRUE, i+10, 50, 10, 10);
	}
	gpk_set_color(widget, gc0, RED_DARK);
	for (i=100;i<180;i+=20) {
		gdk_draw_rectangle(pixmap, gc0, TRUE, i,    180, 10, 10);
		gdk_draw_rectangle(pixmap, gc0, TRUE, i+10, 190, 10, 10);
	}

	// ----- DRAW CHIPS IN DEPOTS -----
	gpk_set_color(widget, gc0, BLACK);
	for (i=0;i<16;i++) {
		if (dt[0][i]!=' ') {
			chip_width=3;
			if (i>7) {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+(i-8)*10, (55-chip_width), chip_width*2, chip_width*2);
			} else {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (45-chip_width), chip_width*2, chip_width*2);
			}
			/*
			chip_width=3;
			if (dt[0][i]=='K') chip_width=5;
			if (dt[0][i]=='Q') chip_width=5;
			if (dt[0][i]=='p') chip_width=2;
			gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (45-chip_width), chip_width*2, chip_width*2);
			if (dt[0][i]=='K') chip_width=3;
			if (dt[0][i]=='Q') chip_width=2;
			if ((dt[0][i]=='K')||(dt[0][i]=='Q')) {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10), chip_width*2, chip_width*2);
			}
			*/
		}
		if (dt[1][i]!=' ') {
			chip_width=3;
			if (i>7) {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+(i-8)*10, (195-chip_width), chip_width*2, chip_width*2);
			} else {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (185-chip_width), chip_width*2, chip_width*2);
			}
			/*
			chip_width=3;
			if (dt[1][i]=='K') chip_width=5;
			if (dt[1][i]=='Q') chip_width=5;
			if (dt[1][i]=='p') chip_width=2;
			gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (185-chip_width), chip_width*2, chip_width*2);
			if (dt[1][i]=='K') chip_width=3;
			if (dt[1][i]=='Q') chip_width=2;
			if (dt[1][i]=='K')||(dt[1][i]=='Q')) {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10), chip_width*2, chip_width*2);
			}
			*/
		}
	}
	
	// ----- DRAW CHIPS -----
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if (gt1[i][j]!=' ') {
				chip_width=3;
				if (gc1[i][j]==2) {
					gpk_set_color(widget, gc0, RED);
				} else {
					gpk_set_color(widget, gc0, NAVY_BLUE);
				}
				if (gt1[i][j]=='K') chip_width=5;
				if (gt1[i][j]=='Q') chip_width=5;
				if (gt1[i][j]=='p') chip_width=2;
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10), chip_width*2, chip_width*2);

				if (gc1[i][j]==2) {
					gpk_set_color(widget, gc0, RED_2);
				} else {
					gpk_set_color(widget, gc0, NAVY_BLUE_2);
				}
				if (gt1[i][j]=='K') chip_width=3;
				if (gt1[i][j]=='Q') chip_width=2;
				if ((gt1[i][j]=='K')||(gt1[i][j]=='Q')) {
					gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10),
                                                           chip_width*2, chip_width*2);
				}
			}
		}
	}
	//gdk_draw_line(pixmap, widget->style->black_gc, 15+xx, 8, 15+xx, 245);
	//gdk_draw_line(pixmap, widget->style->black_gc, 10, 15+yy, 253, 15+yy);
	//gdk_draw_line(pixmap, widget->style->black_gc, 100+xx, 8, 100+xx, 245);
	//gdk_draw_line(pixmap, widget->style->black_gc, 10, 80+yy, 253, 80+yy);

	// ----- DRAW X e Y ROBOT AXIS -----
	gpk_set_color(widget, gc0, BLACK);
	gdk_draw_line(pixmap, gc0, 15+xx*10, 8, 15+xx*10, 245);
	gdk_draw_line(pixmap, gc0, 10, 15+yy*10, 253, 15+yy*10);

	// ----- DRAW ELEVATION EFFECT ON Z AXIS -----
	gpk_set_color(widget, gc0, YELLOW);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 15+xx*10-zz, 15+yy*10-zz, zz*2, zz*2);
	
	// ----- DRAW TAKEN CHIP -----
	if (ct>0) {
		if (ct==1) gpk_set_color(widget, gc0, NAVY_BLUE);
		if (ct==2) gpk_set_color(widget, gc0, RED);
		gdk_draw_rectangle(pixmap, gc0, TRUE, 13+xx*10, 13+yy*10, 5, 5);
	}

	// ----- UPDATE FRAMEBUFFER -----
	GdkRectangle update_rect;
	update_rect.x = 8;
	update_rect.y = 3;
	update_rect.width = 243;
	update_rect.height = 3;
	gtk_widget_draw (widget, &update_rect);
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap, 8, 5, 5, 0, widget->allocation.width, widget->allocation.height);
	
	// ----- FREE -----	
	//g_object_unref(pixmap);   // MALFUNCTIONING
	//gdk_pixmap_unref(pixmap); // MALFUNCTIONING + DEPRECATED
}


/* ***** UPDATE 1 NUMERIC STAT ***** */
void gpk_update_stat_num(int num, int val) {
	
	gchar temp[8];
	gchar target[16];
	
	temp[7]=0;
	target[15]=0;
	snprintf(temp, 7, "%d", val);
	snprintf(target, 15, "entry%d", num);
	GtkWidget* gwidget=lookup_widget(GTK_WIDGET(here), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);
}


/* ***** UPDATE 1 TEXTUAL STAT ***** */
void gpk_update_stat_str(int num, char *str) {
	
	gchar target[16];
	
	target[15]=0;
	snprintf(target, 15, "entry%d", num);
	GtkWidget* gwidget=lookup_widget(GTK_WIDGET(here), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), str);
}


/* ***** UPDATE STAT NETWORK LAG ***** */
void gpk_update_stat_net_lag(double *num) {
	
	gchar temp[16];
	
	temp[15]=0;
	snprintf(temp, 15, "%.3f sec", *num);
	GtkWidget* gwidget=lookup_widget(GTK_WIDGET(here), "entry101");
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);
}


/* ***** UPDATE STAT NETWORK TRAFFIC ***** */
void gpk_update_stat_net_traf(int num) {
	
	gchar temp[16];
	
	temp[15]=0;
	if (num<1000) {
		snprintf(temp, 15, "%d Bytes", num);
	} else if (num<1000000) {
		snprintf(temp, 15, "%4.2f KBytes", (float)num/(float)1000);
	} else if (num<1000000000) {
		snprintf(temp, 15, "%4.2f MBytes", (float)num/(float)1000000);
	} else {
		snprintf(temp, 15, "%4.2f GBytes", (float)num/(float)1000000000);
	}
	GtkWidget* gwidget=lookup_widget(GTK_WIDGET(here), "entry25");
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);
}


/* ***** UPDATE STATS ***** */
void gpk_update_stats(void) {
	gpk_update_stat_num(13, gn1[1]);
	gpk_update_stat_num(14, gn1[2]);
	gpk_update_stat_num(15, gn1[3]);
	gpk_update_stat_num(16, gn1[4]);
	gpk_update_stat_num(17, gn1[5]);

	gpk_update_stat_num(18, gn2[1]);
	gpk_update_stat_num(19, gn2[2]);
	gpk_update_stat_num(20, gn2[3]);
	gpk_update_stat_num(21, gn2[4]);
	gpk_update_stat_num(22, gn2[5]);

	gpk_update_stat_num(1, vittorie[0]);
	gpk_update_stat_num(3, vittorie[1]);
	gpk_update_stat_str(11, status_translation[stato]);
	gpk_update_stat_str(7, ((actual_player==1)?"Black":"Red"));
	gpk_update_stat_num(10, num_mosse);

	gpk_update_stat_num(2, intra_match);
	gpk_update_stat_num(4, intra_compute);
}


/* ***** CHIP-COUNTER ***** */
void gpk_update_num_chips(int who, gchar type) {
	if (who==2) {
		if (type=='K') gn2[0]--;
		if (type=='Q') gn2[1]--;
		if (type=='B') gn2[2]--;
		if (type=='H') gn2[3]--;
		if (type=='T') gn2[4]--;
		if (type=='p') gn2[5]--;
	} else {
		if (type=='K') gn1[0]--;
		if (type=='Q') gn1[1]--;
		if (type=='B') gn1[2]--;
		if (type=='H') gn1[3]--;
		if (type=='T') gn1[4]--;
		if (type=='p') gn1[5]--;
	}
}


/* ***** CHECKMATE ? ***** */
void gpk_check_gameover(void) {
	if ((gn1[0]==0)||(gn2[0]==0)) {
		g_print("\n\nCHECKMATE !!\n\n");
		vittorie[((gn1[0]==0)?0:1)]++;
		stato=0;
		warm_start=1;
		g_print("\nWarm start initiated....\n");
	}
}


/* ***** PAWNS PROMOTION ***** */
void gpk_automata(void) {

	int i;

	for (i=0;i<8;i++) { // ----- RED PAWN PROMOTION -----
		if ((gt[i][0]=='p')&&(gc[i][0]==2)) {
			gt[i][0]='Q';
		}
	}
	for (i=0;i<8;i++) { // ----- BLACK PAWN PROMOTION -----
		if ((gt[i][7]=='p')&&(gc[i][7]==1)) {
			gt[i][7]='Q';
		}
	}
}


/* ***** EXECUTE CASTLING ***** */
void gpk_do_arrocco(int who) {

	if (who==2) { // ----- RED CASTLING -----
		//gt[6][7]=gt[4][7];
		//gt[5][7]=gt[7][7];
		gt[4][7]=' ';
		gt[5][7]='T';
		gt[6][7]='K';
		gt[7][7]=' ';
		gc[4][7]=0;
		gc[5][7]=2;
		gc[6][7]=2;
		gc[7][7]=0;
	}
	if (who==1) { // ----- BLACK CASTLING -----
		//gt[6][0]=gt[4][0];
		//gt[5][0]=gt[7][0];
		gt[4][0]=' ';
		gt[5][0]='T';
		gt[6][0]='K';
		gt[7][0]=' ';
		gc[4][0]=0;
		gc[5][0]=1;
		gc[6][0]=1;
		gc[7][0]=0;
	}
}


/* ***** PUT ONE NEW MOVE IN THE MOVES ARRAY ***** */
void gpk_insert_valid_move(int x, int y) {
	valid_move[valid_move_cnt][0]=x;
	valid_move[valid_move_cnt][1]=y;
	valid_move_cnt++;
}


/* ***** PUT ONE GOOD CAPTURE MOVE IN THE MOVES ARRAY ***** */
void gpk_insert_best_move(int x, int y) {
	best_move[best_move_cnt][0]=x;
	best_move[best_move_cnt][1]=y;
	best_move_cnt++;
}

/* ***** CHECK IF A CHIP WILL BE CAPTURED ***** */
int gpk_check_warning_position(int x, int y) {

	int i,j,k;
	int his_color=((gc[x][y]==1)?2:1);
	int found=0;
	gchar temp[64];


	// For each opponent chip...
	temp[63]=0;
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if (gc[i][j]==his_color) {
				
				// ...look if he can capture
				gpk_compute_best_moves(i,j);
				if (best_move_cnt>0) {
					for (k=0;k<best_move_cnt;k++) {
						if ((best_move[k][0]==x)&&(best_move[k][1]==y)) {
							found=1;
							snprintf(temp, 63, "\n--- CHECK ---\n\n");
							g_print(temp);
						}
					}
				}
			}
		}
	}
	return found;
}


/* ***** POPULATE CHIPS POSITIONS ARRAY ***** */
int gpk_find_chips(int actual_player) {

	int i,j,cnt;
	int warning=0;
	gchar temp[64];
	
	// INIT
	temp[63]=0;
	cnt=0;
	
	// Mush move king ?
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if ((gt[i][j]=='K')&&(gc[i][j]==actual_player)) {
				if (gpk_check_warning_position(i,j)) {  
					valid_chip[0][0]=i;
					valid_chip[0][1]=j;
					cnt=1;
					warning=1;
				}
			}
		}
	}
	

	// Record all my chips
	if (!warning) {
		for (i=0;i<8;i++) {
			for (j=0;j<8;j++) {
				if ((gt[i][j]!=' ')&&(gc[i][j]==actual_player)) {
					valid_chip[cnt][0]=i;
					valid_chip[cnt][1]=j;
					cnt++;
				}
			}
		}

		// Record the first capture (if any)
		/*
		for (i=0;i<cnt;i++) {
			n=gpk_compute_best_moves(valid_chip[i][0], valid_chip[i][1]);
			if (n>0) {
				valid_chip[0][0]=valid_chip[i][0];
				valid_chip[0][1]=valid_chip[i][1];
				return 1;
			} else {
			}
		}
		*/
	}
	snprintf(temp, 31, "Possible moves: %d\n\n", cnt);
	g_print(temp);
	return cnt;
}


/* ***** DUMP MY CHIPS ***** */
void gpk_dump_valid_chips(void) {

	int i;
	gchar temp[32];

	temp[31]=0;
	snprintf(temp, 31, "Dump movable chips: %d\n", valid_chip_cnt);
	g_print(temp);
	for (i=0;i<valid_chip_cnt;i++) {
		snprintf(temp, 31, "[%d]-[%d]\n", valid_chip[i][0], valid_chip[i][1]);
		g_print(temp);
	}
	g_print ("\n");
}


/* ***** DUMP MOVES ARRAY ***** */
void gpk_dump_valid_moves(void) {

	int i;
	gchar temp[32];

	temp[31]=0;
	snprintf(temp, 31, "Dump valid moves: %d\n", valid_move_cnt);
	g_print(temp);
	for (i=0;i<valid_move_cnt;i++) {
		snprintf(temp, 31, "[%d]-[%d]\n", valid_move[i][0], valid_move[i][1]);
		g_print(temp);
	}
	g_print ("\n");
}


/* ***** DUMP CAPTURING MOVES ARRAY ***** */
void gpk_dump_best_moves(void) {

	int i;
	gchar temp[32];

	temp[31]=0;
	snprintf(temp, 31, "Dump capturing targets: %d\n", best_move_cnt);
	g_print(temp);
	for (i=0;i<best_move_cnt;i++) {
		snprintf(temp, 31, "[%d]-[%d]\n", best_move[i][0], best_move[i][1]);
		g_print(temp);
	}
	g_print ("\n");
}


/* ***** BACKUP ALL BOARD (colors + chip types) IN TEMPORARY MATRIX ***** */
void gpk_backup_board(void) {

	int i,j;

	intra_match++;
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			bc2[i][j]=gc[i][j];
			bt[i][j]=gt[i][j];
		}
	}
}


/* ***** BACKUP BEST MOVES IN TEMPORARY VECTORS ***** */
void gpk_backup_moves(void) {

	int i;

	for (i=0;i<best_move_cnt;i++) {
		temp_best_move[i][0]=best_move[i][0];
		temp_best_move[i][1]=best_move[i][1];
	}
	temp_best_move_cnt=best_move_cnt;
}



/* ***** BACKUP VALID MOVES IN TEMPORARY VECTORS ***** */
void gpk_backup_moves2(void) {

	int i;

	for (i=0;i<valid_move_cnt;i++) {
		temp_valid_move[i][0]=valid_move[i][0];
		temp_valid_move[i][1]=valid_move[i][1];
	}
	for (i=0;i<valid_chip_cnt;i++) {
		temp_valid_chip[i][0]=valid_chip[i][0];
		temp_valid_chip[i][1]=valid_chip[i][1];
	}
	temp_valid_move_cnt=valid_move_cnt;
	temp_valid_chip_cnt=valid_chip_cnt;
}


/* ***** RECOVERY ALL BOARD (colors + chip types + moves) FROM TEMPORARY MATRIX ***** */
void gpk_recovery_board(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc[i][j]=bc2[i][j];
			gt[i][j]=bt[i][j];
		}
	}
}


/* ***** RECOVERY BEST MOVES FROM TEMPORARY VECTORS ***** */
void gpk_recovery_moves(void) {

	int i;

	best_move_cnt=temp_best_move_cnt;
	for (i=0;i<best_move_cnt;i++) {
		best_move[i][0]=temp_best_move[i][0];
		best_move[i][1]=temp_best_move[i][1];
	}
}


/* ***** RECOVERY VALID MOVES FROM TEMPORARY VECTORS ***** */
void gpk_recovery_moves2(void) {

	int i;

	valid_move_cnt=temp_valid_move_cnt;
	valid_chip_cnt=temp_valid_chip_cnt;
	for (i=0;i<valid_move_cnt;i++) {
		valid_move[i][0]=temp_valid_move[i][0];
		valid_move[i][1]=temp_valid_move[i][1];
	}
	for (i=0;i<valid_chip_cnt;i++) {
		valid_chip[i][0]=temp_valid_chip[i][0];
		valid_chip[i][1]=temp_valid_chip[i][1];
	}
}


/* ***** SAVE COLORS IN TEMPORARY MATRIX ***** */
void gpk_backup_colors(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc2[i][j]=gc[i][j];
		}
	}
}


/* ***** SAVE COLORS IN SECOND TEMPORARY MATRIX ***** */
void gpk_backup_colors2(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			bc2[i][j]=gc[i][j];
		}
	}
}


/* ***** RECOVER COLORS FROM TEMPORARY MATRIX ***** */
void gpk_recovery_colors(void) {

	gchar temp[32];
	int i,j;

	gpk_dump_chessboard();
	snprintf(temp, 31, "Recovery Colors 1 (%d)\n", stato);
	g_print (temp);
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc[i][j]=((gc2[i][j]>7)?0:gc2[i][j]); // recovery all (but temporary) colors
		}
	}
	gpk_dump_chessboard();
}


/* ***** RECOVER COLORS FROM SECOND TEMPORARY MATRIX ***** */
void gpk_recovery_colors2(void) {

	gchar temp[32];
	int i,j;

	gpk_dump_chessboard();
	snprintf(temp, 31, "Recovery Colors 2 (%d)\n", stato);
	g_print (temp);
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc[i][j]=bc2[i][j];
		}
	}
	gpk_dump_chessboard();
	gpk_refresh_colors_after_hilight();
	gpk_dump_chessboard();
}


/* ***** SWAP COLOR MATRIX ***** */
void gpk_alternate_colors(void) {

	int i;

	for (i=0;i<valid_move_cnt;i++) {
		if (valid_move[i][1]==10) {        // ----- BLACK CASTLING -----
			gc[4][0]=4;
			gc[5][0]=4;
			gc[6][0]=4;
			gc[7][0]=4;
			arrocco_started=1;
		} else if (valid_move[i][1]==11) { // ----- RED CASTLING -----
			gc[4][7]=4;
			gc[5][7]=4;
			gc[6][7]=4;
			gc[7][7]=4;
			arrocco_started=1;
		} else {                           // ----- NORMAL MOVES -----
			if (gc[valid_move[i][0]][valid_move[i][1]]==0) {
				gc[valid_move[i][0]][valid_move[i][1]]=3; // YELLOW COLOR ON POSSIBLE MOVES
			} else {
				gc[valid_move[i][0]][valid_move[i][1]]=5; // GREEN COLOR ON CAPTURING MOVES
			}
		}
	}
	gc[mossa[0]][mossa[1]]=7; // ORANGE COLOR ON ACTUAL POSITION
}


/* ***** VERIFY IF ONE MOVE IS VALID ***** */
int gpk_check_this_move(int x, int y) {

	int i;
	
	for (i=0;i<28;i++) {
		if ((valid_move[i][0]==x)&&(valid_move[i][1]==y)) {
			return 1;
		}
	}
	return 0;
}


/* ***** CHECKBOARD CHIPS DUMP ***** */
void gpk_dump_chessboard(void) {
	
	int i,j;
	gchar temp[32];
	
	temp[31]=0;
	g_print ("\n");
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if (gc[j][i]==1) {
				g_print("\033[34m");
			} else if (gc[j][i]==2) {
				g_print("\033[31m");
			} else if (gc[j][i]>2) {
				g_print("\033[32m");
			} else {
				g_print("\033[0m");
			}
			snprintf(temp, 31, "%c-%d ", gt[j][i], gc[j][i]);
			g_print(temp);
		}
		g_print ("\n");
	}
	g_print("\033[0m\n");
}


/* ***** CHECKBOARD CLEAR AND REFRESH ***** */
void gpk_update_chessboard(void) {
	
	int i,j;
	
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gpk_set_checkboard(i, j, gt[i][j], gc[i][j]);
		}
	}
}


/* ***** REMOVE OLD MOVES FROM HISTORY COMBOBOX ***** */
void gpk_clear_history(void) {
	
	GtkTextBuffer *buffer;

	GtkWidget *gwidget=lookup_widget(GTK_WIDGET(here), "textview1");
	buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(gwidget));
	gtk_text_buffer_set_text(buffer, " \n", -1);
}


/* ***** HISTORY COMBOBOX UPDATER AND COLOR ALTERNATOR ***** */
void gpk_update_history(gchar *ttt) {
	
	GtkTextBuffer *buffer;
	GtkTextIter start,end;

	GtkWidget *gwidget=lookup_widget(GTK_WIDGET(here), "textview1");
	buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(gwidget));
	gtk_text_buffer_get_iter_at_offset(buffer, &start, num_mosse*5);
	gtk_text_buffer_insert(buffer, &start, ttt, -1);
	gtk_text_buffer_get_iter_at_offset (buffer, &start, num_mosse*5);
	gtk_text_buffer_get_iter_at_offset (buffer, &end, (num_mosse+1)*5);
	if ((num_mosse%2)==0) {
		gtk_text_buffer_apply_tag (buffer, tag, &start, &end);
	} else {
		gtk_text_buffer_apply_tag (buffer, tag2, &start, &end);
	}
}


/* ***** CLEAR CHAT COMBOBOX ***** */
void gpk_clear_chat(void) {
	
	GtkTextBuffer *buffer;

	GtkWidget *gwidget=lookup_widget(GTK_WIDGET(here), "textview2");
	buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(gwidget));
	gtk_text_buffer_set_text(buffer, " \n", -1);
}


/* ***** CHAT COMBOBOX UPDATER ***** */
void gpk_update_chat(gchar *ttt) {
	
	GtkTextBuffer *buffer;
	GtkTextIter start,end;

	GtkWidget *gwidget=lookup_widget(GTK_WIDGET(here), "textview2");
	buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(gwidget));
	gtk_text_buffer_get_iter_at_offset(buffer, &start, num_mosse*5);
	gtk_text_buffer_insert(buffer, &start, ttt, -1);
	gtk_text_buffer_get_iter_at_offset(buffer, &start, num_mosse*5);
	gtk_text_buffer_get_iter_at_offset(buffer, &end, (num_mosse+1)*5);
}


/* ***** INJECT A DEVELOPMENT BOARD IN CHECKBOARD VARS ***** */
void gpk_inject_board(void) {

	int i,j;

	// RESET POSIZIONS AND COLORs
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gt[i][j]=' ';
			gt1[i][j]=' ';
			gc[i][j]=0;
			gc1[i][j]=0;
		}
	}		
	
	// PUT CHIPS IN MATRIX STARTING POSITIONS
	gt[4][2]='K';
	gt1[4][2]='K';

	gc[4][2]=1;
	gc1[4][2]=1;

	gt[0][0]='K';
 	gt1[0][0]='K';
	gt[6][7]='Q';
 	gt1[6][7]='Q';
	gt[2][7]='Q';
 	gt1[2][7]='Q';
	gt[6][5]='Q';
 	gt1[6][5]='Q';
	gt[2][5]='Q';
 	gt1[2][5]='Q';

	gc[0][0]=2;
 	gc1[0][0]=2;
	gc[6][7]=2;
 	gc1[6][7]=2;
	gc[2][7]=2;
 	gc1[2][7]=2;
	gc[6][5]=2;
 	gc1[6][5]=2;
	gc[2][5]=2;
 	gc1[2][5]=2;

	}


/* ***** GUI REFRESH - THREADS RESET ***** */
void gpk_game_reset(GtkButton *button) {

	GtkTextBuffer *buffer;
	int i,j,tmp;

	// Get I/O permissions
	GtkToggleButton *widget_a1=(GtkToggleButton *)lookup_widget(GTK_WIDGET(button), "checkbutton19");
	GtkToggleButton *widget_a2=(GtkToggleButton *)lookup_widget(GTK_WIDGET(button), "checkbutton17");
	if (1) {
	//if (getuid()!=0) {                                     // Old Parallel Port Code DISABLED
		gtk_toggle_button_set_active(widget_a1, FALSE);
		gtk_toggle_button_set_active(widget_a2, FALSE);
		//g_print ("Not root.... Lpt I/O Access Disabled !!\n");
	} else {
		gtk_toggle_button_set_active(widget_a1, TRUE);
		gtk_toggle_button_set_active(widget_a2, TRUE);
		if (lpt_perm_taken!=1) {
			if (ioperm(base_addr1,1,1)) fprintf(stderr, "Couldn't get the io-port 0x%x (no root?)\n", base_addr1), exit(1);
			gpk_outb(0, 0);
			lpt_perm_taken=1;
		}
	}
	
	// Setup Textarea History e Setup Array (ONLY ONCE)
	if (!tag_flag) {
		GtkWidget *gwidget=lookup_widget(GTK_WIDGET(button), "textview1");
		buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(gwidget));
		tag2=gtk_text_buffer_create_tag(buffer, "red_foreground", "foreground", "red", NULL);
		tag=gtk_text_buffer_create_tag(buffer, "blue_foreground", "foreground", "blue", NULL);
		tag_flag++;
		
		tmp=1;
		for (i=0;i<25;i++) {
			translate_bit[i]=tmp;
			tmp*=2;
			if (tmp>8) { tmp=1; }
		}
	}

	// SETUP TRANSLATIONS
	for (i=0;i<32;i++) {
		snprintf(status_translation[i], 31, " ");
	}
	snprintf(status_translation[0], 31, "Ready");
	snprintf(status_translation[1], 31, "Move");
	snprintf(status_translation[10], 31, "Ready");
	snprintf(status_translation[11], 31, "Move");
	snprintf(status_translation[20], 31, "Ready");
	snprintf(status_translation[21], 31, "Move");
	snprintf(status_translation[22], 31, "Ready");
	snprintf(status_translation[23], 31, "Move");

	// INIT COLORS
	col_lgrey.red   = 60000;
	col_lgrey.green = 60000;
	col_lgrey.blue  = 60000;
	col_red.red   = 65535;
	col_red.green = 0;
	col_red.blue  = 0;
	col_black.red   = 0;
	col_black.green = 0;
	col_black.blue  = 0;
	col_yellow.red   = 65535;
	col_yellow.green = 65535;
	col_yellow.blue  = 0;
	col_white.red   = 65535;
	col_white.green = 65535;
	col_white.blue  = 65535;
	col_purple.red   = 65535;
	col_purple.green = 0;
	col_purple.blue  = 65535;
	col_green.red   = 47000;
	col_green.green = 65535;
	col_green.blue  = 0;
	col_orange.red   = 65535;
	col_orange.green = 32768;
	col_orange.blue  = 0;
	col_cyan.red   = 40000;
	col_cyan.green = 44000;
	col_cyan.blue  = 65535;
	col_grey.red   = 26000;
	col_grey.green = 26000;
	col_grey.blue  = 26000;

	// INIT VARS
	here=(GtkWidget *)button;
	stato=0;
	stato_r==20;
	intra_match=0;
	intra_compute=0;
	timer_on=0;
	timer[0]=1;
	timer[1]=1;
	arrocco_flag[0]=1;
	arrocco_flag[1]=1;
	arrocco_started=0;
	actual_player=1;
	num_mosse=0;
	xx=0;
	yy=0;
	zz=0;
	tt=' ';
	ct=0;
	mangiate[0]=0;
	mangiate[1]=0;
	xd[0]=9;
	yd[0]=3;
	xd[1]=9;
	yd[1]=17;

	// CHIP NUMBERS RESET
	gn1[0]=1;
	gn1[1]=1;
	gn1[2]=2;
	gn1[3]=2;
	gn1[4]=2;
	gn1[5]=8;
	gn2[0]=1;
	gn2[1]=1;
	gn2[2]=2;
	gn2[3]=2;
	gn2[4]=2;
	gn2[5]=8;

	// RESET CHIPS TAKEN DEPOSITS
	for (i=0;i<16;i++) {
		dt[0][i]=' ';
		dt[1][i]=' ';
	}

	// RESET POSIZIONS AND COLORs
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gt[i][j]=' ';
			gt1[i][j]=' ';
			gc[i][j]=0;
			gc1[i][j]=0;
		}
	}		
	
	// PUT CHIPS IN MATRIX STARTING POSITIONS
	gt[0][0]='T';
	gt[1][0]='H';
	gt[2][0]='B';
	gt[3][0]='Q';
 	gt[4][0]='K';
	gt[5][0]='B';
	gt[6][0]='H';
	gt[7][0]='T';
	for (i=0; i<8; i++) {
		gt[i][1]='p';
		gt[i][6]='p';
	}
	gt[0][7]='T';
	gt[1][7]='H';
	gt[2][7]='B';
	gt[3][7]='Q';
	gt[4][7]='K';
	gt[5][7]='B';
	gt[6][7]='H';
	gt[7][7]='T';

	gt1[0][0]='T';
	gt1[1][0]='H';
	gt1[2][0]='B';
	gt1[3][0]='Q';
 	gt1[4][0]='K';
	gt1[5][0]='B';
	gt1[6][0]='H';
	gt1[7][0]='T';
	for (i=0; i<8; i++) {
		gt1[i][1]='p';
		gt1[i][6]='p';
	}
	gt1[0][7]='T';
	gt1[1][7]='H';
	gt1[2][7]='B';
	gt1[3][7]='Q';
	gt1[4][7]='K';
	gt1[5][7]='B';
	gt1[6][7]='H';
	gt1[7][7]='T';

	// PUT STARTING COLORS IN MATRIX
	for (i=0; i<8; i++) {
		gc[i][0]=1;
		gc[i][1]=1;
		gc[i][6]=2;
		gc[i][7]=2;
		gc1[i][0]=1;
		gc1[i][1]=1;
		gc1[i][6]=2;
		gc1[i][7]=2;
	}

	// DRAW CHECKBOARD AND ROBOT AXIS
	gpk_draw_position();
	gpk_draw_position2();
	
	// START ROBOT THREAD
	if (thread2_started==0) {
		pthread_create(&thd2, NULL, gpk_thread2, button);
		g_print ("J-Robot started now....\n");
		thread2_started++;
	}

	// START MESSAGE THREAD
	message_blink_threshold=20;
	snprintf(message_string, 31, "Connecting...."); // Prewrite message for the next action
	if (thread3_started==0) {
		pthread_create(&thd3, NULL, gpk_thread3, button);
		g_print ("Message started now....\n");
		thread3_started++;
	}

	// START NETWORK THREAD
	need_refresh_list=1;
	if (thread4_started==0) {
		pthread_create(&thd4, NULL, gpk_thread4, button);
		g_print ("Network started now....\n");
		thread4_started++;
	}

	// UPDATE HISTORY & CHAT
	//gpk_clear_history(); // moved out to let history there after autoplay and clear only manually
	gpk_clear_chat();
}


/* ***** SET DISABLE BUTTONS NOT SENSITIVE ***** */
void gpk_disable_buttons(void) {

	gchar target[32];

	target[31]=0;

	// Disable history buttons
	snprintf(target, 31, "button73");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button74");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button77");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button78");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button81");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button80");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);

	//Disable community buttons
	snprintf(target, 31, "button106");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button105");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button97");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button98");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button99");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);

	// Disable chat buttons
	snprintf(target, 31, "button100");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button101");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button102");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button103");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button104");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);

	// Disable robot setup
	snprintf(target, 31, "button82");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button83");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
	snprintf(target, 31, "button86");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(here), target), 0);
}


/* ***** FIND AND CHANGE COLORS OF VALID MOVES FROM ACTUAL POSITION ***** */
int gpk_compute_valid_moves(int x, int y) {

	int i,j,k,q,flag;
	int new_valid_move_cnt=0;
	int my_color=((gc[x][y]==1)?1:2); // ***** UNUSED
	int his_color=((gc[x][y]==1)?2:1);
        int temp_move[28][2];
	gchar target[32];
	int need_help=0;
	GtkToggleButton *help;
	
	target[31]=0;
	valid_move_cnt=0;
	if (gc[x][y]==actual_player) { 

                // Reset Vars
                his_positions_cnt=0;
                for (i=0;i<16;i++) {
                        his_positions[i][0]=0;
                        his_positions[i][1]=0;
                }
                for (i=0;i<28;i++) {
                        best_move[i][0]=0;
                        best_move[i][1]=0;
                        valid_move[i][0]=0;
                        valid_move[i][1]=0;
                        temp_move[i][0]=0;
                        temp_move[i][1]=0;
                }

		if (gt[x][y]=='K') { // ------------------- K I N G ----- R E -----------------------
		
			// --- vertical & orizontal ---
			if ((x<7)&&((gc[x+1][y]==his_color)||(gt[x+1][y]==' '))) gpk_insert_valid_move(x+1, y);
			if ((x>0)&&((gc[x-1][y]==his_color)||(gt[x-1][y]==' '))) gpk_insert_valid_move(x-1, y);
			if ((y>0)&&((gc[x][y-1]==his_color)||(gt[x][y-1]==' '))) gpk_insert_valid_move(x, y-1);
			if ((y<7)&&((gc[x][y+1]==his_color)||(gt[x][y+1]==' '))) gpk_insert_valid_move(x, y+1);

			// --- diagonals ---
			if ((x<7)&&(y<7)&&((gc[x+1][y+1]==his_color)||(gt[x+1][y+1]==' '))) gpk_insert_valid_move(x+1, y+1);
			if ((x>0)&&(y>0)&&((gc[x-1][y-1]==his_color)||(gt[x-1][y-1]==' '))) gpk_insert_valid_move(x-1, y-1);
			if ((x<7)&&(y>0)&&((gc[x+1][y-1]==his_color)||(gt[x+1][y-1]==' '))) gpk_insert_valid_move(x+1, y-1);
			if ((x>0)&&(y<7)&&((gc[x-1][y+1]==his_color)||(gt[x-1][y+1]==' '))) gpk_insert_valid_move(x-1, y+1);

			// --- BLACK castling ---
			if ((arrocco_flag[0])&&(gt[5][0]==' ')&&(gt[6][0]==' ')) gpk_insert_valid_move(6, 10);

			// --- RED castling ---
			if ((arrocco_flag[1])&&(gt[5][7]==' ')&&(gt[6][7]==' ')) gpk_insert_valid_move(6, 11);
		}

		if (gt[x][y]=='H') { // ------------------- K N I G T ----- C A V A L L O -----------------------
		
			if ((x<7)&&(y<6)&&((gc[x+1][y+2]==his_color)||(gt[x+1][y+2]==' '))) gpk_insert_valid_move(x+1, y+2);
			if ((x<6)&&(y<7)&&((gc[x+2][y+1]==his_color)||(gt[x+2][y+1]==' '))) gpk_insert_valid_move(x+2, y+1);
			if ((x>0)&&(y>1)&&((gc[x-1][y-2]==his_color)||(gt[x-1][y-2]==' '))) gpk_insert_valid_move(x-1, y-2);
			if ((x>1)&&(y>0)&&((gc[x-2][y-1]==his_color)||(gt[x-2][y-1]==' '))) gpk_insert_valid_move(x-2, y-1);
			if ((x>0)&&(y<6)&&((gc[x-1][y+2]==his_color)||(gt[x-1][y+2]==' '))) gpk_insert_valid_move(x-1, y+2);
			if ((x<6)&&(y>0)&&((gc[x+2][y-1]==his_color)||(gt[x+2][y-1]==' '))) gpk_insert_valid_move(x+2, y-1);
			if ((x<7)&&(y>1)&&((gc[x+1][y-2]==his_color)||(gt[x+1][y-2]==' '))) gpk_insert_valid_move(x+1, y-2);
			if ((x>1)&&(y<7)&&((gc[x-2][y+1]==his_color)||(gt[x-2][y+1]==' '))) gpk_insert_valid_move(x-2, y+1);
		}

		if (gt[x][y]=='p') { // ------------------- P A W N ----- P E D O N E -----------------------
		
			if (gc[x][y]==1) { // ----- BLACK -----
				if ((y<7)&&(gt[x][y+1]==' '))                 gpk_insert_valid_move(x, y+1);   // move down 
				if ((y==1)&&(gt[x][2]==' ')&&(gt[x][3]==' ')) gpk_insert_valid_move(x, 3);     // the first time can go down of 2 positions
				if ((x<7)&&(y<7)&&(gc[x+1][y+1]==2))          gpk_insert_valid_move(x+1, y+1); // capture down-right
				if ((x>0)&&(y<7)&&(gc[x-1][y+1]==2))          gpk_insert_valid_move(x-1, y+1); // capture down-left
			}
			if (gc[x][y]==2) { // ----- RED -----
				if ((y>0)&&(gt[x][y-1]==' '))                 gpk_insert_valid_move(x, y-1);   // move up
				if ((y==6)&&(gt[x][5]==' ')&&(gt[x][4]==' ')) gpk_insert_valid_move(x, 4);     // the first time can go up of 2 positions
				if ((x<7)&&(y>0)&&(gc[x+1][y-1]==1))          gpk_insert_valid_move(x+1, y-1); // capture up-right
				if ((x>0)&&(y>0)&&(gc[x-1][y-1]==1))          gpk_insert_valid_move(x-1, y-1); // capture up-left
			}
		}

		if ((gt[x][y]=='T')||(gt[x][y]=='Q')) { // -------------- ROOK + QUEEN ----- TORRE + REGINA -----------------

			// -- going down --
			flag=0; i=y+1;
			while ((flag==0)&&(i<8)) {
				if (gt[x][i]==' ') gpk_insert_valid_move(x, i);
				if (gt[x][i]!=' ') {
					if (gc[x][i]==his_color) gpk_insert_valid_move(x, i);
					flag++;
				}
				i++;
			}

			// -- going up --
			flag=0; i=y-1;
			while ((flag==0)&&(i>=0)) {
				if (gt[x][i]==' ') gpk_insert_valid_move(x, i);
				if (gt[x][i]!=' ') {
					if (gc[x][i]==his_color) gpk_insert_valid_move(x, i);
					flag++;
				}
				i--;
			}

			// -- slide right --
			flag=0; i=x+1;
			while ((flag==0)&&(i<8)) {
				if (gt[i][y]==' ') gpk_insert_valid_move(i, y);
				if (gt[i][y]!=' ') {
					if (gc[i][y]==his_color) gpk_insert_valid_move(i, y);
					flag++;
				}
				i++;
			}

			// -- slide left --
			flag=0; i=x-1;
			while ((flag==0)&&(i>=0)) {
				if (gt[i][y]==' ') gpk_insert_valid_move(i, y);
				if (gt[i][y]!=' ') {
					if (gc[i][y]==his_color) gpk_insert_valid_move(i, y);
					flag++;
				}
				i--;
			}
		}

		if ((gt[x][y]=='B')||(gt[x][y]=='Q')) { // ---------------- BISHOP + QUEEN ----- ALFIERE + REGINA -----------------

			// -- down right --
			flag=0; i=x+1; j=y+1;
			while ((flag==0)&&(i<8)&&(j<8)) {
				if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
				if (gt[i][j]!=' ') {
					if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
					flag++;
				}
				i++; j++;
			}

			// -- down left --
			flag=0; i=x-1; j=y+1;
			while ((flag==0)&&(i>=0)&&(j<8)) {
				if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
				if (gt[i][j]!=' ') {
					if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
					flag++;
				}
				i--; j++;
			}

			// -- up left --
			flag=0; i=x-1; j=y-1;
			while ((flag==0)&&(i>=0)&&(j>=0)) {
				if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
				if (gt[i][j]!=' ') {
					if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
					flag++;
				}
				i--; j--;
			}

			// -- up right --
			flag=0; i=x+1; j=y-1;
			while ((flag==0)&&(i<8)&&(j>=0)) {
				if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
				if (gt[i][j]!=' ') {
					if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
					flag++;
				}
				i++; j--;
			}
		}


		// Flush suicide moves from the list if not player 1 && hand 1
		help=(GtkToggleButton *)lookup_widget(GTK_WIDGET(here), "checkbutton20");
		need_help=(int)gtk_toggle_button_get_active(help);
		//if (((stato!=20)&&(stato!=21))||(need_help>0)) {
		if (((stato>8)&&(stato<18))||(stato==22)||(need_help>0)) {
			his_positions_cnt=0;
			for (i=0;i<8;i++) {
				for (j=0;j<8;j++) {
					if (gc[i][j]==his_color) {
						his_positions[his_positions_cnt][0]=i; // Get his positions
						his_positions[his_positions_cnt][1]=j;
						his_positions_cnt++;
					}
				}
			}
			gpk_backup_moves2(); // ******* Saves also suicide moves, then starts STRATEGY COMPUTATION:
			g_print("Applying Strategy: [");
			for (i=0;i<valid_move_cnt;i++) { // For each possible my move
				gpk_backup_board();
				gpk_backup_moves();
				gt[valid_move[i][0]][valid_move[i][1]]=gt[x][y]; // Do temporarly my move
				gt[x][y]=' ';
				gc[valid_move[i][0]][valid_move[i][1]]=gc[x][y];
				gc[x][y]=0;
				for (j=0;j<his_positions_cnt;j++) {
					gpk_compute_best_moves(his_positions[j][0], his_positions[j][1]); // Get his response moves
					for (k=0;k<best_move_cnt;k++) {
						if ((best_move[k][0]==valid_move[i][0])&&(best_move[k][1]==valid_move[i][1])) {
							for (q=0;q<best_move_cnt;q++) {
								if ((temp_best_move[q][0]==valid_move[i][0])&&(temp_best_move[q][1]==valid_move[i][1])) {
									temp_best_move[q][0]=9;
									temp_best_move[q][1]=9; // Mark suicide best moves on backup chackboard (it will be recovered)
								}
							}
							valid_move[i][0]=9; // Mark suicide valid moves of the selected chip (on backup board that will be recovered)
							valid_move[i][1]=9;
							if (bc[best_move[k][0]][best_move[k][1]]==2) {
								bc[best_move[k][0]][best_move[k][1]]=20;
								g_print("F");
							} else if (bc[best_move[k][0]][best_move[k][1]]==1) {
								bc[best_move[k][0]][best_move[k][1]]=10;
								g_print("C");
							} else {
								bc[best_move[k][0]][best_move[k][1]]=8;
								g_print("G");
							}
                                        	} else {  // Mark all his possible captures on all my chips (on backup board that will be recovered)
							if (bc[best_move[k][0]][best_move[k][1]]==2) {
								bc[best_move[k][0]][best_move[k][1]]=21;
								g_print("U");
							} else if (bc[best_move[k][0]][best_move[k][1]]==1) {
								bc[best_move[k][0]][best_move[k][1]]=11;
								g_print("P");
							} else {
								bc[best_move[k][0]][best_move[k][1]]=9;
								g_print("A");
							}
						}
                                	}
					g_print("T");
                        	}
				gpk_recovery_moves();
				gpk_recovery_board();
                	}
			g_print("]\n\n");
		}

		new_valid_move_cnt=0;
                for (i=0;i<valid_move_cnt;i++) { // Delete suicide valid moves
			if ((valid_move[i][0]!=9)&&((valid_move[i][0]!=0)||(valid_move[i][1]!=0))) { // TODO - Find bug and remove this shit
               	        //if (valid_move[i][0]!=9) {
                       	        temp_move[new_valid_move_cnt][0]=valid_move[i][0];
                       	        temp_move[new_valid_move_cnt][1]=valid_move[i][1];
                       	        new_valid_move_cnt++;
			}
               	}
		for (i=0;i<new_valid_move_cnt;i++) {
			valid_move[i][0]=temp_move[i][0]; // Copy cleared list
                       	valid_move[i][1]=temp_move[i][1]; 
			//snprintf(target, 31, "VALID COPY Dump:%d (%d,%d)=(%d,%d)\n", i, valid_move[i][0], valid_move[i][1], temp_move[i][0], temp_move[i][1]);
			//g_print(target);
               	}
	}

        // Do last things and return purifyed array count
	if ((new_valid_move_cnt==0)&&(valid_move_cnt>0)) {
		need_refresh_gui=1;
	}
	if (((stato!=20)&&(stato!=21))||(need_help>0)) {
		valid_move_cnt=new_valid_move_cnt;
	}

	//return valid_move_cnt; // ***** DEBUG THIS 8 ROWS CONSIDERING THE 2 FOR UP
	return new_valid_move_cnt;
}


/* ***** REMOVE BEST MOVE NOT VALID (high risk moves) ***** */
void gpk_choose_best_move(void) {

	int i,j,flag;
	int new_best_move_cnt;
        int temp_move[28][2];
	gchar target[64];

	if (best_move_cnt>0) {
		target[63]=0;
	        for (i=0;i<best_move_cnt;i++) { // Mark best moves not found valid
			flag=0;
			g_print("[chk");
        		for (j=0;j<valid_move_cnt;j++) {
				if ((best_move[i][0]==valid_move[j][0])&&(best_move[i][1]==valid_move[j][1])) {
					flag=1;
				}
			}
			if (flag==0) {
				best_move[i][0]=9;
				best_move[i][1]=9;
				g_print(" BAD");
			}
			g_print("] ");
		}
		new_best_move_cnt=0;
		for (i=0;i<best_move_cnt;i++) { // Don't copy "suicide" best moves
			if (best_move[i][0]!=9) {
				temp_move[new_best_move_cnt][0]=best_move[i][0];
				temp_move[new_best_move_cnt][1]=best_move[i][1];
				new_best_move_cnt++;
				g_print("[normal] ");
			}
		}
		for (i=0;i<new_best_move_cnt;i++) {
			best_move[i][0]=temp_move[i][0]; // Refill best movie array
			best_move[i][1]=temp_move[i][1]; 
			snprintf(target, 63, "BEST COPY Dump:%d (%d,%d)=(%d,%d)\n", i, best_move[i][0], best_move[i][1], best_move[i][0], best_move[i][1]);
			g_print(target);
		}
		best_move_cnt=new_best_move_cnt;
		if (best_move_cnt==0) {
			best_move[0][0]=valid_move[0][0];
			best_move[0][1]=valid_move[0][1];
			best_move_cnt=1;
			g_print("[recover] ");
		}
		g_print("\n");
	}
}


/* ***** SEARCH ALL MOVES THAT CAPTURE ***** */
int gpk_compute_best_moves(int x, int y) {

	int i,j,flag;
	int my_color=((gc[x][y]==1)?1:2); // ***** UNUSED
	int his_color=((gc[x][y]==1)?2:1);

	best_move_cnt=0;
	intra_compute++;

	for (i=0;i<28;i++) {
		best_move[i][0]=0;
		best_move[i][1]=0;
	}

	if (gt[x][y]=='K') { // ------------------- R E -----------------------
		
		// --- normali ---
		if ((x<7)&&(gc[x+1][y]==his_color)) gpk_insert_best_move(x+1, y);
		if ((x>0)&&(gc[x-1][y]==his_color)) gpk_insert_best_move(x-1, y);
		if ((y>0)&&(gc[x][y-1]==his_color)) gpk_insert_best_move(x, y-1);
		if ((y<7)&&(gc[x][y+1]==his_color)) gpk_insert_best_move(x, y+1);

		// --- diagonali ---
		if ((x<7)&&(y<7)&&(gc[x+1][y+1]==his_color)) gpk_insert_best_move(x+1, y+1);
		if ((x>0)&&(y>0)&&(gc[x-1][y-1]==his_color)) gpk_insert_best_move(x-1, y-1);
		if ((x<7)&&(y>0)&&(gc[x+1][y-1]==his_color)) gpk_insert_best_move(x+1, y-1);
		if ((x>0)&&(y<7)&&(gc[x-1][y+1]==his_color)) gpk_insert_best_move(x-1, y+1);
	}

	if (gt[x][y]=='H') { // ------------------- C A V A L L O -----------------------
		
		if ((x<7)&&(y<6)&&(gc[x+1][y+2]==his_color)) gpk_insert_best_move(x+1, y+2);
		if ((x<6)&&(y<7)&&(gc[x+2][y+1]==his_color)) gpk_insert_best_move(x+2, y+1);
		if ((x>0)&&(y>1)&&(gc[x-1][y-2]==his_color)) gpk_insert_best_move(x-1, y-2);
		if ((x>1)&&(y>0)&&(gc[x-2][y-1]==his_color)) gpk_insert_best_move(x-2, y-1);
		if ((x>0)&&(y<6)&&(gc[x-1][y+2]==his_color)) gpk_insert_best_move(x-1, y+2);
		if ((x<6)&&(y>0)&&(gc[x+2][y-1]==his_color)) gpk_insert_best_move(x+2, y-1);
		if ((x<7)&&(y>1)&&(gc[x+1][y-2]==his_color)) gpk_insert_best_move(x+1, y-2);
		if ((x>1)&&(y<7)&&(gc[x-2][y+1]==his_color)) gpk_insert_best_move(x-2, y+1);
	}

	if (gt[x][y]=='p') { // ------------------- P E D O N E -----------------------
		
		if (gc[x][y]==1) { // ----- NERO -----
			if ((x<7)&&(y<7)&&(gc[x+1][y+1]==2)) gpk_insert_best_move(x+1, y+1); // mangia a destra
			if ((x>0)&&(y<7)&&(gc[x-1][y+1]==2)) gpk_insert_best_move(x-1, y+1); // mangia a sinistra
		}
		if (gc[x][y]==2) { // ----- ROSSO -----
			if ((x<7)&&(y>0)&&(gc[x+1][y-1]==1)) gpk_insert_best_move(x+1, y-1); // mangia a destra
			if ((x>0)&&(y>0)&&(gc[x-1][y-1]==1)) gpk_insert_best_move(x-1, y-1); // mangia a sinistra
		}
	}

	if ((gt[x][y]=='T')||(gt[x][y]=='Q')) { // ------------------- T O R R E  /  R E G I N A -----------------------

		// -- scende --
		flag=0; i=y+1;
		while ((flag==0)&&(i<8)) {
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_best_move(x, i);
				flag++;
			}
			i++;
		}

		// -- sale --
		flag=0; i=y-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_best_move(x, i);
				flag++;
			}
			i--;
		}

		// -- slide DX --
		flag=0; i=x+1;
		while ((flag==0)&&(i<8)) {
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_best_move(i, y);
				flag++;
			}
			i++;
		}

		// -- slide SX --
		flag=0; i=x-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_best_move(i, y);
				flag++;
			}
			i--;
		}
	}

	if ((gt[x][y]=='B')||(gt[x][y]=='Q')) { // ------------------- A L F I E R E  /  R E G I N A -----------------------

		// -- DOWN DX --
		flag=0; i=x+1; j=y+1;
		while ((flag==0)&&(i<8)&&(j<8)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i++; j++;
		}

		// -- DOWN SX --
		flag=0; i=x-1; j=y+1;
		while ((flag==0)&&(i>=0)&&(j<8)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i--; j++;
		}

		// -- UP SX --
		flag=0; i=x-1; j=y-1;
		while ((flag==0)&&(i>=0)&&(j>=0)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i--; j--;
		}

		// -- UP DX --
		flag=0; i=x+1; j=y-1;
		while ((flag==0)&&(i<8)&&(j>=0)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i++; j--;
		}
	}
	return best_move_cnt;
}


/* ***** RESET COLOR NUMBERS TO POSSESSORS ***** */
void gpk_refresh_colors_after_hilight(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if ((gc[i][j]==21)||(gc[i][j]==20)) {
				gc[i][j]=2;
			} else if ((gc[i][j]==11)||(gc[i][j]==10)) {
				gc[i][j]=1;
			} else if (gc[i][j]>7) {
				gc[i][j]=0;
			} else {
				// Leave matrix as is
			}
		}
	}
}


/* ***** IS THE KING WITHOUT MOVES (not suicide) ***** */
int gpk_check_king_locked(int x, int y) {

	int i,j;
	int flag=0;
	int his_color=((gc[pre_x][pre_y]==1)?1:2);
	gchar temp[64];

	his_positions_cnt=0;
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if (gc[i][j]==his_color) {
				his_positions[his_positions_cnt][0]=i; // Get his positions
				his_positions[his_positions_cnt][1]=j;
				his_positions_cnt++;
			}
		}
	}
	//gpk_backup_moves2();
	gpk_backup_moves();
	for (i=0;i<his_positions_cnt;i++) {
		snprintf(temp, 63, "HIS POSITIONS [%d-%d]\n", his_positions[i][0], his_positions[i][1]);
		g_print(temp);
		gpk_compute_best_moves(his_positions[i][0], his_positions[i][1]);
		for (j=0;j<best_move_cnt;j++) {
			snprintf(temp, 63, "BEST MOVE [%d-%d]\n", best_move[j][0], best_move[j][1]);
			g_print(temp);
			if ((best_move[j][0]==x)&&(best_move[j][1]==y)) { // Can the other player capture my king?
				flag=1;
			}
		}
	}
	gpk_recovery_moves();
	//gpk_recovery_moves2();
	return flag;
}


/* ***** FIND MOVES & DO IT ***** */
void gpk_actor_game(int x, int y) {
	
	gchar temp[64];
	int changed=0;
	int checkmate_flag;
	int my_color=((gc[pre_x][pre_y]==1)?1:2);

	// Refresh if needed before all...
	if (need_refresh_gui) {
		gpk_refresh_colors_after_hilight();
		gpk_update_chessboard();
		need_refresh_gui=0;
	}
	
	// ***** MAIN *****
	temp[63]=0;
	snprintf(temp, 63, "----------------------------------------- ACTOR MOVE [%d-%d]\n\n", x, y);
	g_print(temp);
	if ((stato==1)||(stato==21)) { // -0- IF CHOOSE TO MOVE ANOTHER CHIP (not the one jush clicked) RESET STATE
		if (!gpk_check_this_move(x,y)) {
			gpk_recovery_colors();
			stato--;
		}
	}
	if ((stato==0)||(stato==10)||(stato==20)||(stato==22)) {                      // -1- TAKE UP THE CHIP.....
		if (gc[x][y]!=actual_player) {
			snprintf(temp, 63, "WARN: Actual:%d Click:%d Stato:%d\n", actual_player, gc[x][y], stato);
			g_print(temp);
			gpk_dump_chessboard();
		} else {
			//snprintf(temp, 63, "HAND: Actual:%d Click:%d Stato:%d\n", actual_player, gc[x][y], stato);
			//g_print(temp);
			while (stato_r>0) usleep(DELAY_PRE_COMPUTE); // wait robot movement to be finished
			if (gpk_compute_valid_moves(x,y)>0) {        // ....however search valid moves
				gpk_compute_best_moves(x,y);
				mossa[0]=x;
				mossa[1]=y;
				gpk_backup_colors();
				gpk_alternate_colors();
				stato++;
				changed++;
			} else {
				if (gt[x][y]=='K') {
					valid_chip_cnt=gpk_find_chips(actual_player);
					if (valid_chip_cnt>0) {
						g_print ("FOUND VALID !!\n");
						gpk_dump_valid_chips();
						checkmate_flag=gpk_check_king_locked(x,y);
						if (checkmate_flag==1) {
							vittorie[my_color]++;
							warm_start=1;
							g_print("\nCHECKMATE in 1 MOVE !!\n");
							g_print("\nWarm start initiated....\n\n");
						} else {
							// do diffent move
						}
					} else {
						g_print ("NO VALID CHIP !! ....You would not be here !\n");
					}
				}
			}
		}
	}
	if (((stato==1)||(stato==11)||(stato==21)||(stato==23))&&(changed==0)) {      // -2- PUT IT ON THE CHECKBOARD
		//gpk_dump_valid_moves();
		//gpk_dump_best_moves();
		while (stato_r>0) usleep(DELAY_PRE_COMPUTE2); // wait robot movement to be finished 
		if (stato==23) usleep(DELAY_SHOW_OTHER_MOVE); // wait to let human see the move (if needed)
		gpk_recovery_colors();
		if (arrocco_started) {
			if ((x==6)&&(y==0)) y=10; // conventional position for castling
			if ((x==6)&&(y==7)) y=11; // conventional position for castling
			arrocco_started=0;
		}
		if (gpk_check_this_move(x,y)) {
			if (y>9) {                            // if castling required, do it
				gpk_do_arrocco(gc[mossa[0]][mossa[1]]);
			} else {                              // ....else do normal move
				if (gt[mossa[0]][mossa[1]]=='K') arrocco_flag[(gc[mossa[0]][mossa[1]]==2)?1:0]=0;
				if (gc[x][y]>0) gpk_update_num_chips((gc[mossa[0]][mossa[1]]), gt[x][y]);
				// ----- update array -----
				gt[x][y]=gt[mossa[0]][mossa[1]];
				gc[x][y]=gc[mossa[0]][mossa[1]];
				gt[mossa[0]][mossa[1]]=' ';
				gc[mossa[0]][mossa[1]]=0;
				// ----- move robot -----
				x1=mossa[0];
				y1=mossa[1];
				x2=x;
				y2=y;
				stato_r=5;
				actual_player++;
			}
			snprintf(temp, 63, "%c%c%d \n", gt[x][y], 65+x2, y2);
			gpk_update_history(temp); // insert last move into the history
		} else {
			if (stato==21) stato--;
			//actual_player++;
		}
		if (stato==20) {
			//stato++;
		} else if (stato==21) {
			stato++;
		} else if (stato==23) {
			stato=20;
		} else {
			stato--;
		}
		gpk_automata();
		num_mosse++;
		//actual_player++;
		if (actual_player>2) actual_player=1;
	}
	gpk_update_stats();
	gpk_check_gameover();
	gpk_update_chessboard();

	// Saves previous move
	pre_x=x;
	pre_y=y;
}


/* ***** Wrapper to put characters on checkboard ***** */
void gpk_set_checkboard(int x, int y, gchar val, int colore) {
	
	gchar temp[3];
	gchar target[12];
	GtkStyle *style;

	temp[2]=0;
	target[11]=0;
	snprintf(temp, 2, "%c", val);
	snprintf(target, 11, "entry%d", (short)(26+x+(y*8)));

	GtkWidget* gwidget=lookup_widget(GTK_WIDGET(here), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);

	// ***** Draw white squares on checkboard
	if (((x-y)%2)==0) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_white;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Draw black squares on checkboard
	if (((x-y)%2)!=0) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_lgrey;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw red color chip fonts
	if (gc[x][y]==2) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->text[0]=col_red;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw black color chip fonts
	if (gc[x][y]==1) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->text[0]=col_black;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw possible moves on background
	if (colore==3) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_yellow;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw possible captures on background
	if (colore==4) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_purple;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw possible captures on background
	if (colore==5) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_green;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw actual position on background
	if (colore==7) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_orange;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw chip in danger
	if ((colore==8)||(colore==10)||(colore==20)) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_cyan;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}

	// ***** Dwaw chips in danger
	if ((colore==9)||(colore==11)||(colore==21)) {
		style=gtk_widget_get_style(gwidget);
		style=gtk_style_copy(style); //Saw this one in a mailing-list :)
		style->base[0]=col_grey;
		gtk_widget_set_style(gwidget, style);
		g_object_unref(style);
	}
}


/* ***** RESET BUTTON ***** */
void on_button65_clicked (GtkButton *button, gpointer user_data) {

	vittorie[0]=0;
	vittorie[1]=0;
	if (cold_start) {
		g_print("\nCold start initiated....\n");
		gpk_game_reset((GtkButton *)here);
		cold_start=0;
	} else {
		warm_start=1;
		g_print("\nWarm start initiated....\n");
	}
	if (stato==0) {
		gpk_clear_history(); // moved here to let history stay after autoplay and clear only manually
	} else {
		stato=0;
	}
	gpk_update_chessboard();
	gpk_update_stats();
}


/* ***** 2 PLAYERS BUTTON ***** */
void on_button66_clicked (GtkButton *button, gpointer user_data) {

	gchar target[32];

	target[31]=0;
	snprintf(target, 31, "button67");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 0);
	snprintf(target, 31, "button68");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 0);
	timer_on=1;
	message_blink_threshold=10;
	snprintf(message_string, 31, "Play free chess");
	gpk_clear_history();
	g_print ("Call 2 Players\n");
}


/* ***** 1 PLAYER BUTTON ***** */
void on_button67_clicked (GtkButton *button, gpointer user_data) {

	gchar target[32];

	target[31]=0;
	snprintf(target, 31, "button66");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 0);
	snprintf(target, 31, "button68");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 0);
	timer_on=1;
	message_blink_threshold=10;
	snprintf(message_string, 31, "You are black");
	g_print ("Call 1 Players\n");
	stato=20;
	gpk_clear_history();
	if (thread_started==0) {
		pthread_create(&thd1, NULL, gpk_thread, button);
		g_print ("Thread started now....\n");
		thread_started++;
	}
	g_print ("Thread informaed to start game....\n");
}


/* ***** 0 PLAYERS BUTTON ***** */
void on_button68_clicked (GtkButton *button, gpointer user_data) {

	gchar target[32];

	target[31]=0;
	snprintf(target, 31, "button66");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 0);
	snprintf(target, 31, "button67");
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 0);
	timer_on=1;
	message_blink_threshold=10;
	snprintf(message_string, 31, "Autoplay");
	g_print ("Call 0 Players\n");
	stato=10;
	gpk_clear_history();
	if (thread_started==0) {
		pthread_create(&thd1, NULL, gpk_thread, button);
		g_print ("Thread started now....\n");
		thread_started++;
	}
	g_print ("Thread informaed to start game....\n");
}


/* ***** AUTOPLAY THREAD ***** */
void *gpk_thread(void *arg) {

	int r,x,y;

	g_print ("==> Autoplay Thread UP and RUNNING !!\n");
	while (1) {
		while ((stato==9)||(stato==10)||(stato==22)||(stato==23)) {
			g_print ("\n\n\n\n\n\n\n***************************\n");
			if (actual_player==1) {
				g_print ("*  START HAND - PLAYER 1  *\n");
			} else {
				g_print ("*  START HAND - PLAYER 2  *\n");
			}
			g_print ("***************************\n\n");
			valid_chip_cnt=gpk_find_chips(actual_player);
			if (valid_chip_cnt>0) {
				r=(int)(random()%valid_chip_cnt);
				x=valid_chip[r][0];
				y=valid_chip[r][1];
				g_print ("Waiting Robot, if needed...\n");
				while (stato_r>0) usleep(DELAY_THREAD_WAIT_MOVE);  //             !! WARNING DEADLOCK !!
				g_print ("Done.\n\n");
				gdk_threads_enter();
				gpk_actor_game(x, y);
				gdk_threads_leave();
				usleep(DELAY_THREAD_BETWEEN_MOVE);
				if ((best_move_cnt>0)&&(valid_move_cnt==0)) g_print(" ==> PASSO <==\n");
				if ((best_move_cnt>0)&&(valid_move_cnt>0)) {                         // ----- search good (capturing) move -----
					//r=(int)(random()%best_move_cnt);
					gpk_choose_best_move();
					while (stato_r>0) usleep(DELAY_THREAD_WAIT_MOVE2);  //    !! WARNING DEADLOCK !!
					gdk_threads_enter();
					gpk_actor_game(best_move[0][0], best_move[0][1]);
					gdk_threads_leave();
					g_print (" ==> GOOD found <==\n");
				} else if (valid_move_cnt>0) {                  // -------- ...if no good move, take a random valid one --------
					r=(int)(random()%valid_move_cnt);
					while (stato_r>0) usleep(DELAY_THREAD_WAIT_MOVE3);  //    !! WARNING DEADLOCK !!
					gdk_threads_enter();
					gpk_actor_game(valid_move[r][0], valid_move[r][1]);
					gdk_threads_leave();
					g_print (" ==> VALID found <==\n");
				} else { 					// -------- ...if not any move, try recoverying suicide moves ------- 
					gpk_recovery_moves2();
					g_print ("RECOVERED:\n");
					gpk_dump_valid_moves();
					if (valid_move_cnt>0) {
						g_print (" ==> ( MISS RECOVERED ) <==\n");
						r=(int)(random()%valid_move_cnt);
						while (stato_r>0) usleep(DELAY_THREAD_WAIT_MOVE3);  //    !! WARNING DEADLOCK !!
						gdk_threads_enter();
						gpk_actor_game(valid_move[r][0], valid_move[r][1]);
						gdk_threads_leave();
					} else {
						g_print (" ==> REAL MISS <==\n");
					}
				}
			} else {
				g_print ("==> NO CHIPS (You would not be here !!)\n");
			}
		}
		usleep(DELAY_THREAD_BETWEEN_PLAYERS);
	}
	g_print ("==> THREAD END (You would not be here !!)\n");
}


/* ***** DRAW AREA / MOVE ROBOT THREAD ***** */
void *gpk_thread2(void *arg) {
	
	int stato_old;                              // stato[1..49] ==> move    stato[50..99] ==> capture 
	int player=0;
	gchar temp[35];

	temp[34]=0;
	g_print ("J-Robot Thread UP and RUNNING !!\n");
	while (1) {
		while (stato_r>0) {                               // ----- state: wait -----
			stato_old=stato_r;
			if (stato_old==5) {                       // ----- state: capture check -----
				if (gt1[x2][y2]==' ') {
					stato_r=7;
				} else {
					stato_r=50;
					player=((gc1[x1][y1]==1)?1:0);
				}
			}
			
			// ----- under here: movement states -----
			
			if (stato_old==7) {                       // ----- state: acceleration -----
				if (xx<(x1+xo)) xx++;
				if (yy<(y1+yo)) yy++;
				if ((xx==(x1+xo))&&(yy==(y1+yo))) stato_r=10;
			}
			if (stato_old==10) {                      // ----- state: go down on chip -----
				if (zz<zo) { zz++; } else { stato_r=11; }
			}
			if (stato_old==11) {                      // ----- state: take the chip -----
				tt=gt1[x1][y1];
				ct=gc1[x1][y1];
				gt1[x1][y1]=' ';
				gc1[x1][y1]=0;
				stato_r=12;
			}
			if (stato_old==12) {                      // ----- state: take up chip -----
				if (zz>0) { zz--; } else { stato_r=15; }
			}
			if (stato_old==15) {                      // ----- state: take away the chip -----
				if (xx!=(x2+xo)) {
					if (xx<(x2+xo)) { xx++; } else { xx--; }
				}
				if (yy!=(y2+yo)) {
					if (yy<(y2+yo)) { yy++; } else { yy--; }
				}
				if ((xx==(x2+xo))&&(yy==(y2+yo))) stato_r=16;
			}
			if (stato_old==16) {                      // ----- state: take down the chip -----
				if (zz<zo) { zz++; } else { stato_r=18; }
			}
			if (stato_old==18) {                      // ----- state: leave the chip -----
				gt1[x2][y2]=tt;
				gc1[x2][y2]=ct;
				tt=' ';
				ct=0;
				stato_r=19;
			}
			if (stato_old==19) {                      // ----- state: move up -----
				if (zz>0) { zz--; } else { stato_r=20; }
			}
			if (stato_old==20) {                      // ----- state: return -----
				if (xx>0) xx--;
				if (yy>0) yy--;
				if ((xx==0)&&(yy==0)) stato_r=30; // ----- state: the end -----
			}
			
			// ----- under here: capturing states -----
			
			if (stato_old==50) {                       // ----- state: acceleration -----
				if (xx<(x2+xo)) xx++;
				if (yy<(y2+yo)) yy++;
				if ((xx==(x2+xo))&&(yy==(y2+yo))) stato_r=55;
			}
			if (stato_old==55) {                      // ----- state: go down to the chip -----
				if (zz<zo) { zz++; } else { stato_r=60; }
			}
			if (stato_old==60) {                      // ----- state: take the chip -----
				tt=gt1[x2][y2];
				ct=gc1[x2][y2];
				gt1[x2][y2]=' ';
				gc1[x2][y2]=0;
				stato_r=65;
			}
			if (stato_old==65) {                      // ----- state: take up the chip -----
				if (zz>0) { zz--; } else { stato_r=70; }
			}
			if (stato_old==70) {                       // ----- state: take the chip to the storage -----
				if (xx!=(xd[player])) {
					if (xx<(xd[player])) { xx++; } else { xx--; }
				}
				if (yy!=(yd[player])) {
					if (yy<(yd[player])) { yy++; } else { yy--; }
				}
				if ((xx==(xd[player]))&&(yy==(yd[player]))) stato_r=75;
			}
			if (stato_old==75) {                      // ----- state: take down the chip to the storage -----
				if (zz<zo) { zz++; } else { stato_r=80; }
			}
			if (stato_old==80) {                      // ----- state: leave the chip -----
				dt[player][mangiate[player]]=tt;
				tt=' ';
				ct=0;
				stato_r=85;
			}
			if (stato_old==85) {                      // ----- state: go up -----
				if (zz>0) { zz--; } else { stato_r=90; }
			}
			if (stato_old==90) {                       // ----- state: go to source -----
				if (xx!=(x1+xo)) {
					if (xx<(x1+xo)) { xx++; } else { xx--; }
				}
				if (yy!=(y1+yo)) {
					if (yy<(y1+yo)) { yy++; } else { yy--; }
				}
				if ((xx==(x1+xo))&&(yy==(y1+yo))) stato_r=95;
			}
			if (stato_old==95) {                      // ----- state: go down to the chip -----
				if (zz<zo) { zz++; } else { stato_r=100; }
			}
			if (stato_old==100) {                      // ----- state: take the chip -----
				tt=gt1[x1][y1];
				ct=gc1[x1][y1];
				gt1[x1][y1]=' ';
				gc1[x1][y1]=0;
				stato_r=105;
			}
			if (stato_old==105) {                      // ----- state: take up the chip -----
				if (zz>0) { zz--; } else { stato_r=110; }
			}
			if (stato_old==110) {                       // ----- state: take away the chip -----
				if (xx!=(x2+xo)) {
					if (xx<(x2+xo)) { xx++; } else { xx--; }
				}
				if (yy!=(y2+yo)) {
					if (yy<(y2+yo)) { yy++; } else { yy--; }
				}
				if ((xx==(x2+xo))&&(yy==(y2+yo))) stato_r=115;
			}
			if (stato_old==115) {                      // ----- state: move down the chip -----
				if (zz<zo) { zz++; } else { stato_r=120; }
			}
			if (stato_old==120) {                      // ----- state: leave the chip -----
				gt1[x2][y2]=tt;
				gc1[x2][y2]=ct;
				tt=' ';
				ct=0;
				stato_r=125;
			}
			if (stato_old==125) {                      // ----- state: go up -----
				if (zz>0) { zz--; } else { stato_r=130; }
			}
			if (stato_old==130) {                      // ----- state: return -----
				if (xx>0) xx--;
				if (yy>0) yy--;
				if ((xx==0)&&(yy==0)) stato_r=135;
			}
			if (stato_old==135) {                      // ----- state: update stats -----
				xd[player]++;
				if (xd[player]>16) {
					yd[player]++;
					xd[player]=9;
				}
				mangiate[player]++;
				stato_r=30;                        // ----- state: the end -----
			}
			
			/* ***** PAUSE BUTTON PRESSED ***** */
			while (game_paused==1) {
				usleep(DELAY_THREAD_WAIT_MOVE);
			}
			
			/* ***** GUI UPDATE ***** */
			gdk_threads_enter();
				gpk_update_stat_num(1, stato_r);
				gpk_update_stat_num(5, xx);
				gpk_update_stat_num(6, yy);
				gpk_draw_position();
				gpk_draw_position2();
			gdk_threads_leave();
			
			//snprintf(temp, 34, "Stato_r[%d][%d]: %d\n", xx, yy, stato_r);
			//g_print (temp);
			/*
			if (stato_r>49) {
				usleep(DELAY_THREAD2_BETWEEN_STATES_SLOW);
			} else {
				usleep(DELAY_THREAD2_BETWEEN_STATES);
			}
			*/
			usleep(DELAY_THREAD2_BETWEEN_STATES);
			if (stato_r==30) stato_r=0;              // ----- state: quiet -----
			
			/* ***** MOVE JOSHUA !! ***** */
			//outb((int)((xx%256)|((yy%256)*16)), 0);
			gdk_threads_enter();
				gpk_outb(translate_bit[xx], 0);
				gpk_outb(translate_bit[yy], 1);
				gpk_outb(translate_bit[zz], 2);
			gdk_threads_leave();
		}
		usleep(DELAY_THREAD2_BETWEEN_STATES2);
	}
	g_print ("THREAD END (You would not be here !!)\n");
}


/* ***** MESSAGES / TIMERS THREAD ***** */
void *gpk_thread3(void *arg) {
	
	int tot_time;
	gchar temp1[16];
	gchar temp2[16];
	gchar temp3[16];
	
	temp1[15]=0;
	temp2[15]=0;
	temp3[15]=0;
	g_print ("Message Thread UP and RUNNING !!\n");
	while (1) {
		
		/* ***** BLINK MESSAGES ***** */
		gdk_threads_enter();
			if (message_hilight>0) {
				gpk_update_stat_str(24, "");
				message_hilight=0;
			} else {
				gpk_update_stat_str(24, message_string);
				message_hilight=1;
			}
		gdk_threads_leave();
		if (message_blink_threshold>0) {
			message_blink_threshold--;
			usleep(DELAY_THREAD3_BETWEEN_HILIGHT_FAST);
		} else {
			usleep(DELAY_THREAD3_BETWEEN_HILIGHT_SLOW);
			
			/* ***** UPDATE TIMERS ***** */
			if (timer_on) {
				timer[actual_player-1]++;
			}
		}
		tot_time=timer[0]+timer[1];
		snprintf(temp1, 15, "%02d:%02d", (int)timer[0]/60, (int)timer[0]%60);
		snprintf(temp2, 15, "%02d:%02d", (int)timer[1]/60, (int)timer[1]%60);
		snprintf(temp3, 15, "%02d:%02d", (int)tot_time/60, (int)tot_time%60);
		gdk_threads_enter();
			gpk_update_stat_str(98, temp1);
			gpk_update_stat_str(99, temp2);
			gpk_update_stat_str(100, temp3);
		gdk_threads_leave();
	}
	g_print ("THREAD END (You would not be here !!)\n");
}


/* ***** CHAT MESSAGE FETCH - THREAD ***** */
void *gpk_thread4(void *arg) {

	gchar target[32];
	GtkButton *button=(GtkButton *)arg;
	
	g_print ("Network Thread UP and RUNNING !!\n");
	while (1) {
	//	g_print ("==> gpk_thread4 <==\n");
		//gpk_curl_fetch("http://joshua.gipoco.com/msgserv.php?cmd=ping&session=1122334412345678&nick=hhkk&ver=0.1.5a", 2);
		gdk_threads_enter();
			/*
			if (chunk.size>0) {
				n=chunk.size;
				if (n>999) { n=999; }
				memset(chunk.memory, 0, 1000);
				snprintf(msg, n, "%s\n", chunk.memory);
				gpk_update_chat(msg);
			}
			*/
		//gpk_curl_free(2);
			if (need_refresh_list>0) {
				gpk_clear_community_treeview();
				snprintf(message_string, 31, "Choose num players"); // Prewrite message for the next phase
	 			gpk_populate_community_treeview();
				need_refresh_list=0;
			}
			if (warm_start>0) {
				g_print("\nWarm start pending !!");
				if (stato_r==0) {
					g_print("\nWarm start EXECUTED !!\n\n");
					gpk_game_reset(button);
					gpk_update_stats();
					gpk_update_chessboard();
					target[31]=0;
					snprintf(target, 31, "button66");
					gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 1);
					snprintf(target, 31, "button67");
					gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 1);
					snprintf(target, 31, "button68");
					gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button), target), 1);
					warm_start=0;
				}
			}
		gdk_threads_leave();
		usleep(DELAY_THREAD4_NETWORK_WAIT);
	}
	g_print ("THREAD END (You would not be here !!)\n");
}


/* ***** HOOK SIGNALS CALLBACKS ***** */
gboolean on_entry1_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { g_print ("Call GG\n"); return FALSE; }

gboolean on_entry26_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 0);
		return FALSE;
	}
}
gboolean on_entry27_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 0);
		return FALSE;
	}
}
gboolean on_entry28_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 0);
		return FALSE;
	}
}
gboolean on_entry29_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 0);
		return FALSE;
	}
}
gboolean on_entry30_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 0);
		return FALSE;
	}
}
gboolean on_entry31_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 0);
		return FALSE;
	}
}
gboolean on_entry32_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 0);
		return FALSE;
	}
}
gboolean on_entry33_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 0);
		return FALSE;
	}
}


gboolean on_entry34_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 1);
		return FALSE;
	}
}
gboolean on_entry35_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 1);
		return FALSE;
	}
}
gboolean on_entry36_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 1);
		return FALSE;
	}
}
gboolean on_entry37_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 1);
		return FALSE;
	}
}
gboolean on_entry38_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 1);
		return FALSE;
	}
}
gboolean on_entry39_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 1);
		return FALSE;
	}
}
gboolean on_entry40_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 1);
		return FALSE;
	}
}
gboolean on_entry41_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 1);
		return FALSE;
	}
}


gboolean on_entry42_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 2);
		return FALSE;
	}
}
gboolean on_entry43_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 2);
		return FALSE;
	}
}
gboolean on_entry44_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 2);
		return FALSE;
	}
}
gboolean on_entry45_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 2);
		return FALSE;
	}
}
gboolean on_entry46_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 2);
		return FALSE;
	}
}
gboolean on_entry47_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 2);
		return FALSE;
	}
}
gboolean on_entry48_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 2);
		return FALSE;
	}
}
gboolean on_entry49_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 2);
		return FALSE;
	}
}


gboolean on_entry50_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 3);
		return FALSE;
	}
}
gboolean on_entry51_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 3);
		return FALSE;
	}
}
gboolean on_entry52_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 3);
		return FALSE;
	}
}
gboolean on_entry53_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 3);
		return FALSE;
	}
}
gboolean on_entry54_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 3);
		return FALSE;
	}
}
gboolean on_entry55_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 3);
		return FALSE;
	}
}
gboolean on_entry56_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 3);
		return FALSE;
	}
}
gboolean on_entry57_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 3);
		return FALSE;
	}
}


gboolean on_entry58_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 4);
		return FALSE;
	}
}
gboolean on_entry59_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 4);
		return FALSE;
	}
}
gboolean on_entry60_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 4);
		return FALSE;
	}
}
gboolean on_entry61_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 4);
		return FALSE;
	}
}
gboolean on_entry62_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 4);
		return FALSE;
	}
}
gboolean on_entry63_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 4);
		return FALSE;
	}
}
gboolean on_entry64_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 4);
		return FALSE;
	}
}
gboolean on_entry65_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 4);
		return FALSE;
	}
}


gboolean on_entry66_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 5);
		return FALSE;
	}
}
gboolean on_entry67_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 5);
		return FALSE;
	}
}
gboolean on_entry68_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 5);
		return FALSE;
	}
}
gboolean on_entry69_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 5);
		return FALSE;
	}
}
gboolean on_entry70_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 5);
		return FALSE;
	}
}
gboolean on_entry71_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 5);
		return FALSE;
	}
}
gboolean on_entry72_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 5);
		return FALSE;
	}
}
gboolean on_entry73_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 5);
		return FALSE;
	}
}


gboolean on_entry74_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 6);
		return FALSE;
	}
}
gboolean on_entry75_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 6);
		return FALSE;
	}
}
gboolean on_entry76_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 6);
		return FALSE;
	}
}
gboolean on_entry77_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 6);
		return FALSE;
	}
}
gboolean on_entry78_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 6);
		return FALSE;
	}
}
gboolean on_entry79_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 6);
		return FALSE;
	}
}
gboolean on_entry80_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 6);
		return FALSE;
	}
}
gboolean on_entry81_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 6);
		return FALSE;
	}
}


gboolean on_entry82_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(0, 7);
		return FALSE;
	}
}
gboolean on_entry83_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(1, 7);
		return FALSE;
	}
}
gboolean on_entry84_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(2, 7);
		return FALSE;
	}
}
gboolean on_entry85_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(3, 7);
		return FALSE;
	}
}
gboolean on_entry86_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(4, 7);
		return FALSE;
	}
}
gboolean on_entry87_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(5, 7);
		return FALSE;
	}
}
gboolean on_entry88_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(6, 7);
		return FALSE;
	}
}
gboolean on_entry89_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (stato_r==0) {
		gpk_actor_game(7, 7);
		return FALSE;
	}
}






gboolean on_drawingarea1_button_press_event(GtkWidget *here, GdkEventButton *event, gpointer user_data) {
	g_print ("Push test\n");
	return FALSE;
}


gboolean on_button69_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	if (game_paused==0) {
		game_paused=1;
		g_print ("Game Paused.\n");
	} else {
		game_paused=0;
		g_print ("Recovering...\n");
	}
	return FALSE;
}


gboolean on_button70_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	gpk_update_chessboard();
	gpk_draw_position();
	gpk_draw_position2();
	g_print ("Refresh.\n");
	return FALSE;
}


gboolean on_button71_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	gpk_inject_board();
	gpk_update_chessboard();
	gpk_draw_position();
	gpk_draw_position2();
	g_print ("Development Board Injected !\n");
	return FALSE;
}


gboolean on_button72_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data) {

	GtkWidget *dialog;

	g_print ("About dialog...\n");
	dialog=gtk_message_dialog_new(GTK_WINDOW(here), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
					"Joshua Chess Robot 0.1.6a\n\nCopyRight 2011 by GiPOCO\nReleased under GNU GPL V3");
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	g_print ("Closed.\n");
	return FALSE;
}


void on_button82_clicked (GtkButton *button, gpointer user_data) {
	g_print ("Click test\n");
}


void on_button83_clicked (GtkButton *button, gpointer user_data) {

	GtkTreeModel *model;
	GtkTreeIter new_row;
	GtkWidget *treeview=lookup_widget(GTK_WIDGET(button), "treeview1");

	model=gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	gtk_list_store_append(GTK_LIST_STORE(model), &new_row);
	gtk_list_store_set(GTK_LIST_STORE(model), &new_row, COL_NAME, "GiPOCO", COL_AGE, "WAIT", -1);

	g_print ("Set 2\n");
}


void on_button86_clicked (GtkButton *button, gpointer user_data) {
	g_print ("Set 3\n");
}


/* ***** CURL WEB FETCHER ***** */
void gpk_curl_fetch(gchar *ttt, int debug) {
	
	gchar tmp[32];
	
	tmp[31]=0;
        network_lag=malloc(sizeof(double));
	snprintf(tmp, 31, "(%d) Requested XML\n", debug);
	g_print(tmp);
	while (gpk_curl_lock==1) {
		usleep(DELAY_CURL_LOCK);
	}
	snprintf(tmp, 31, "(%d) Fetching XML\n", debug);
	g_print(tmp);
	gpk_curl_lock=1;
	chunk.memory=NULL;
	chunk.size=0;
	curl_global_init(CURL_GLOBAL_ALL);
	curl_handle=curl_easy_init();
	curl_easy_setopt(curl_handle, CURLOPT_NOPROGRESS, 1);
	curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
	curl_easy_setopt(curl_handle, CURLOPT_CONNECTTIMEOUT, 3);
	//curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, 3);
	//curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT_MS, 3);
	curl_easy_setopt(curl_handle, CURLOPT_URL, ttt);
	curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
	curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *)&chunk);
	curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "libcurl-agent/1.0");
	curl_easy_perform(curl_handle);
	curl_easy_cleanup(curl_handle);
	curl_easy_getinfo(curl_handle, CURLINFO_TOTAL_TIME, network_lag);
	gpk_update_stat_net_lag(network_lag);
	network_traf+=chunk.size+gpk_strlen(ttt)+45+100; // Update traffic with (download + upload + headers and overhead)
	gpk_update_stat_net_traf(network_traf);
	snprintf(tmp, 31, "(%d) Received XML\n", debug);
	g_print(tmp);
	free(network_lag);
}


/* ***** FREE CURL MEMORY ***** */
void gpk_curl_free(int debug) {
	
	gchar tmp[32];
	
	tmp[31]=0;
	if (chunk.memory) {
		memset(chunk.memory, 0, chunk.size);
		free(chunk.memory);
	}
	gpk_curl_lock=0;
	snprintf(tmp, 31, "(%d) Free XML\n", debug);
	g_print(tmp);
}


/* ***** CHAT MESSAGE INJECT ***** */
void on_button98_clicked(GtkButton *button, gpointer user_data) {

/*
	gchar *msg;
	gchar url[512];

	msg=gtk_entry_get_text(gwidget);
	snprintf(url, 512, "http://joshua.gipoco.com/msgserv.php?cmd=talk&nick=test1&msg=%s", msg);
	g_print ("==> on_button98_clicked <==\n");
	gpk_curl_fetch(url, 3);
	gpk_update_stat_str(button, 102, "");
	gpk_curl_free(3);
	g_print ("Chat !!\n");

*/
}

